@@ -16,23 +16,18 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include <gtk/gtk.h>
+#include "gnuconfig.h"
 
+#include "i18n.h"
 #include "game.h"
+#include "xqf-ui.h"
 #include "history.h"
 #include "dialogs.h"
 #include "utils.h"
 #include "config.h"
 #include "addserver.h"
 #include "srv-prop.h"
-
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(string) gettext(string)
-#else
-#define _(string) (string)
-#endif
-
+#include "pref.h"
 
 static struct history *server_history = NULL;
 
@@ -45,6 +40,7 @@ static void server_combo_activate_callback (GtkWidget *widget, gpointer data) {
   enter_server_result = strdup_strip (gtk_entry_get_text (
                                 GTK_ENTRY (GTK_COMBO (server_combo)->entry)));
   history_add (server_history, enter_server_result);
+
   config_set_string ("/" CONFIG_FILE "/Add Server/game", 
                                                       type2id (*server_type));
 }
@@ -55,31 +51,13 @@ static void select_server_type_callback (GtkWidget *widget,
   *server_type = type;
 }
 
-
-static GtkWidget *create_server_type_menu (void) {
-  GtkWidget *menu;
-  GtkWidget *menu_item;
-  int i;
- 
-  menu = gtk_menu_new ();
-
-  for (i = 0; i < GAMES_TOTAL; i++) {
-    menu_item = gtk_menu_item_new ();
-    gtk_menu_append (GTK_MENU (menu), menu_item);
-
-    gtk_container_add (GTK_CONTAINER (menu_item), game_pixmap_with_label (i));
-
-    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-                 GTK_SIGNAL_FUNC (select_server_type_callback), (gpointer) i);
-
-    gtk_widget_show (menu_item);
-  }
-
-  return menu;
-}
-
-
-char *add_server_dialog (enum server_type *type) {
+/** dialog to prompt user for server type and address
+ * @param type pointer where to store the selected type. if UNKNOWN_SERVER,
+ *             preset type will be read from hitory file
+ * @param addr preset string value for address field, NULL for nothing
+ * @returns address string or NULL if user pressed cancel. string must be freed
+ */
+char *add_server_dialog (enum server_type *type, const char* addr) {
   GtkWidget *window;
   GtkWidget *main_vbox;
   GtkWidget *option_menu;
@@ -87,18 +65,26 @@ char *add_server_dialog (enum server_type *type) {
   GtkWidget *label;
   GtkWidget *button;
   GtkWidget *hseparator;
-  char *typestr;
+
+  g_return_val_if_fail(type != NULL, NULL);
 
   enter_server_result = NULL;
   server_type = type;
+ 
+  if(*type == UNKNOWN_SERVER)
+  {
+    char *typestr;
 
-  typestr = config_get_string ("/" CONFIG_FILE "/Add Server/game");
-  if (typestr) {
-    *type = id2type (typestr);
-    g_free (typestr);
-  }
-  else {
-    *type = QW_SERVER;
+    // Get last game type added (stored in server_combo_activate_callback)
+    typestr = config_get_string ("/" CONFIG_FILE "/Add Server/game");
+   
+    if (typestr) {
+      *type = id2type (typestr);
+      g_free (typestr);
+    }
+    else {
+      *type = 0; // Set to first game
+    }
   }
 
   window = dialog_create_modal_transient_window (_("Add Server"), 
@@ -135,16 +121,15 @@ char *add_server_dialog (enum server_type *type) {
   gtk_widget_grab_focus (GTK_COMBO (server_combo)->entry);
   gtk_widget_show (server_combo);
 
-  if (server_history->items)
-    combo_set_vals (server_combo, server_history->items, "");
+  combo_set_vals (server_combo, server_history->items, addr);
 
   /* Server Type Option Menu */
 
-  option_menu = gtk_option_menu_new ();
+  option_menu = create_server_type_menu (*type,
+		      create_server_type_menu_filter_configured,
+		      GTK_SIGNAL_FUNC(select_server_type_callback));
+
   gtk_box_pack_start (GTK_BOX (hbox), option_menu, FALSE, FALSE, 0);
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), 
-                                                  create_server_type_menu ());
-  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), *type);
   gtk_widget_show (option_menu);
 
   gtk_widget_show (hbox);
