@@ -44,6 +44,7 @@ extern int h_errno;	/* h_errno */
 #include "dns.h"
 #include "host.h"
 #include "utils.h"
+#include "debug.h"
 
 #ifndef MAXHOSTNAMELEN
 # define MAXHOSTNAMELEN		64
@@ -134,6 +135,8 @@ static void master_sigchld_handler (int signum) {
   int pid;
   int status;
 
+  debug(3,"master_sigchld_handler(%d)",signum);
+
   while ((pid = waitpid (WAIT_ANY, &status, WNOHANG)) > 0);
 }
 
@@ -783,6 +786,8 @@ void dns_lookup (const char *str) {
   char host[MAXHOSTNAMELEN + 1];
   int len;
 
+  debug(8,"dns_lookup(%s)",str);
+
   len = strlen (str);
   if (len <= MAXHOSTNAMELEN) {
     strcpy (host, str);
