@@ -16,6 +16,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gnuconfig.h"
+
 #include <sys/types.h>
 #include <regex.h>
 #include <stdio.h>	/* FILE, fopen, fclose, fprintf... */
@@ -26,6 +28,7 @@
 
 #include <gtk/gtk.h>
 
+#include "i18n.h"
 #include "xqf.h"
 #include "pref.h"
 #include "pixmaps.h"
@@ -34,8 +37,6 @@
 #include "flt-player.h"
 #include "filter.h"
 #include "debug.h"
-#include "gnuconfig.h"
-
 
 #define REGCOMP_FLAGS 	(REG_EXTENDED | REG_NOSUB | REG_ICASE)
 
@@ -77,7 +78,7 @@ static const char *mode_names[3] = {
 };
 
 
-int player_filter (struct server *s, struct server_filter_vars *vars) {
+int player_filter (struct server *s) {
   /* The 'vars' is ignored in this function, however, since we need it
      for applying the server filter, we will put it in the arguments. */
 
@@ -503,6 +504,11 @@ static int pattern_clist_event_callback (GtkWidget *widget, GdkEvent *event) {
 static void pattern_clist_select_row_callback (GtkWidget *widget, 
                                  int row, int column, GdkEventButton *event) {
   sync_pattern_data ();
+
+  // for some reason this function is called when a row gets deleted, would
+  // segfault later when row was the last one
+  if(row>=g_slist_length(curplrs)) return;
+
   current_row = row;
 
   pattern_clist_sync_selection ();
@@ -675,7 +681,7 @@ static GtkWidget *player_filter_pattern_editor (void) {
   group = NULL;
 
   for (i = 0; i < 3; i++) {
-    mode_buttons[i] = gtk_radio_button_new_with_label (group, mode_names[i]);
+    mode_buttons[i] = gtk_radio_button_new_with_label (group, _(mode_names[i]));
     group = gtk_radio_button_group (GTK_RADIO_BUTTON (mode_buttons[i]));
 
     gtk_signal_connect (GTK_OBJECT (mode_buttons[i]), "clicked",
@@ -745,7 +751,7 @@ void player_filter_page (GtkWidget *notebook) {
   GtkWidget *pixmap;
   GtkWidget *button;
   GtkWidget *peditor;
-  char *titles[5] = { "", "", "", N_("Mode"), N_("Pattern") };
+  char *titles[5] = { "", "", "", _("Mode"), _("Pattern") };
   int i;
 
   page_hbox = gtk_hbox_new (FALSE, 8);
@@ -797,11 +803,15 @@ void player_filter_page (GtkWidget *notebook) {
   gtk_widget_show (scrollwin);
 
   gtk_widget_ensure_style (pattern_clist);
+
+/*FIXME_GTK2: style->font not working with gtk2*/
+#ifndef USE_GTK2
   i = MAX (pixmap_height (group_pix[0].pix), 
 	      pattern_clist->style->font->ascent + 
 	      pattern_clist->style->font->descent + 1);
 
   gtk_clist_set_row_height (GTK_CLIST (pattern_clist), i);
+#endif
 
   /* Buttons */
 
