@@ -28,18 +28,24 @@
 #include "launch.h"
 #include "pixmaps.h"
 
-#define	GAME_CONNECT		0x0001
-#define	GAME_RECORD		0x0002
-#define	GAME_SPECTATE		0x0004
-#define	GAME_PASSWORD		0x0008
-#define	GAME_RCON		0x0010
 
-#define	GAME_QUAKE1_PLAYER_COLORS	0x0100
-#define	GAME_QUAKE1_SKIN		0x0200
+// game->flags
+enum {
+  GAME_CONNECT			= 0x0001,
+  GAME_RECORD			= 0x0002,
+  GAME_SPECTATE			= 0x0004,
+  GAME_PASSWORD			= 0x0008,
+  GAME_RCON			= 0x0010,
+  GAME_ADMIN			= 0x0020,
+  GAME_QUAKE1_PLAYER_COLORS	= 0x0100,
+  GAME_QUAKE1_SKIN		= 0x0200,
+  GAME_QUAKE3_MASTERPROTOCOL	= 0x0400, // master server protocol version is in games_data["masterprotocol"]
+  GAME_LAUNCH_HOSTPORT		= 0x0800, // use hostport rule as port when launching
+};
 
 struct game {
   enum server_type type;
-  int flags;
+  unsigned long flags;
 
   char *name;
   unsigned short default_port;
@@ -49,9 +55,10 @@ struct game {
   char *qstat_str;
   char *qstat_option;
   char *qstat_master_option;
+  const char* icon; // xpm symbol name
   struct pixmap *pix;
 
-  struct player * (*parse_player) (char *tokens[], int num);
+  struct player * (*parse_player) (char *tokens[], int num, struct server *s);
   void (*parse_server) (char *tokens[], int num, struct server *s);
   void (*analyze_serverinfo) (struct server *s);
   int (*config_is_valid) (struct server *s);
@@ -60,10 +67,29 @@ struct game {
   GList * (*custom_cfgs) (char *dir, char *game);
   void (*save_info) (FILE *f, struct server *s);
 
+  /* map functions */
+  /** determine installed maps, destroys previous data */
+  void (*init_maps)(enum server_type);
+  /** return true if s->map is installed, false otherwise */
+  gboolean (*has_map)(struct server* s);
+  /** acquire image data, function allocates space in buf, returns size. buf
+   * must be freed by caller */
+  size_t (*get_mapshot)(struct server* s, guchar** buf);
+
+  char *arch_identifier;
+  enum CPU (*identify_cpu) (struct server *s, const char *versionstr);
+  enum OS (*identify_os) (struct server *s, char *versionstr);
+
+  char* suggest_commands;
+
   char *cmd;
   char *dir;
   char *real_dir;
   char *game_cfg;
+  GData *games_data;
+  GSList *custom_args;
+ /** game specific private data */
+  void *pd;
 };
 
 extern struct game games[];
@@ -72,4 +98,11 @@ extern	enum server_type id2type (const char *id);
 extern	const char *type2id (enum server_type type);
 extern	GtkWidget *game_pixmap_with_label (enum server_type type);
 
+// retreive game specific value that belongs to key, do not free return value!
+const char* game_get_attribute(enum server_type type, const char* key);
+// set game specific key/value pair, value is _not_ copied
+const char* game_set_attribute(enum server_type type, const char* key, char* value);
+
+void init_games(void);
+
 #endif /* __GAME_H__ */
