@@ -16,6 +16,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gnuconfig.h"
+
 #include <sys/types.h>	/* chmod, waitpid, kill */
 #include <string.h>	/* memchr, strlen */
 #include <unistd.h>	/* execvp, fork, _exit, chdir, read, close */
@@ -24,9 +26,11 @@
 #include <sys/wait.h>	/* waitpid */
 #include <errno.h>	/* errno */
 #include <signal.h>     /* kill, signal... */
+#include <stdlib.h>     /* setenv */
 
 #include <gtk/gtk.h>
 
+#include "i18n.h"
 #include "xqf.h"
 #include "game.h"
 #include "pref.h"
@@ -41,13 +45,6 @@
 # define WAIT_ANY	-1
 #endif
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(string) gettext(string)
-#else
-#define _(string) (string)
-#endif
-
 #define CLIENT_ERROR_BUFFER	256
 #define	CLIENT_ERROR_MSG_HEAD	"<XQF ERROR> "
 
@@ -76,9 +73,7 @@ static void dialog_failed (char *func, char *arg) {
 
 static void client_free (struct running_client *cl) {
 
-#ifdef DEBUG
-  fprintf (stderr, "client detached (pid:%d, fd:%d)\n", cl->pid, cl->fd);
-#endif
+  debug(3, "client detached (pid:%d, fd:%d)\n", cl->pid, cl->fd);
 
   if (cl->fd >= 0) {
     gdk_input_remove (cl->tag);
@@ -130,7 +125,14 @@ static void client_sigchild_handler (int signum) {
   GSList *list;
   struct running_client *cl;
 
+  debug(3,"client_sigchild_handler(%d)",signum);
+
   while ((pid = waitpid (WAIT_ANY, &status, WNOHANG)) > 0) {
+    debug(4,"client_sigchild_handler() -- pid %d, status %d",pid,WEXITSTATUS(status));
+    if(WIFSIGNALED(status)&& WTERMSIG(status)==SIGSEGV)
+    {
+      debug(0,"*** SEGFAULT pid %d ***",pid);
+    }
     for (list = clients; list; list = list->next) {
       cl = (struct running_client *) list->data;
       if (cl->pid == pid) {
@@ -193,7 +195,7 @@ static void client_input_callback (struct running_client *cl, int fd,
 
       pid = cl->pid;  /* save PID value */
       client_detach (cl);
-      kill (pid, SIGTERM);
+      if(pid) kill (pid, SIGTERM); // kill(0) would kill this process too!
     }
     else {
       client_detach (cl);
@@ -215,15 +217,13 @@ static void client_attach (pid_t pid, int fd, struct server *s) {
   cl->server = s;
   server_ref (s);
 
-#ifdef DEBUG
-  fprintf (stderr, "client attached (pid:%d)\n", cl->pid);
-#endif
+  debug (3, "client attached (pid:%d)", cl->pid);
 
   clients = g_slist_append (clients, cl);
 }
 
 
-int client_launch_exec (int forkit, char *dir, char *argv[], 
+int client_launch_exec (int forkit, char *dir, char* argv[], 
                                                            struct server *s) {
   int pid;
   int pipefds[2];
@@ -237,14 +237,16 @@ int client_launch_exec (int forkit, char *dir, char *argv[],
     }
   }
 
-  if (get_debug_level()){
-    char **argptr = argv;
-    fprintf (stderr, "client_launch_exec() -- EXEC> ");
-    while (*argptr)
-      fprintf (stderr, "%s ", *argptr++);
-    fprintf (stderr, "\n");
+  if (get_debug_level() || dontlaunch)
+  {
+    char* cmdline = g_strjoinv(" # ",argv);
+    debug(0,"%s",cmdline);
+    g_free(cmdline);
   }
 
+  if (dontlaunch)
+    return -1;
+
   if (forkit) {
 
     if (pipe (pipefds) < 0) {
@@ -273,15 +275,14 @@ int client_launch_exec (int forkit, char *dir, char *argv[],
     else {	/* child */
       close (pipefds[0]);
 
+      if( s->flags & SERVER_PUNKBUSTER )
+	      setenv("XQF_SERVER_ANTICHEAT","1",1);
+
       execvp (argv[0], argv);
   
       g_snprintf (msg, CLIENT_ERROR_BUFFER, "%sexec(%s) failed: %s", 
                           CLIENT_ERROR_MSG_HEAD, argv[0], g_strerror (errno));
 
-#ifdef DEBUG
-      fprintf (stderr, "%s\n", msg);
-#endif
-
       write (pipefds[1], msg, strlen (msg) + 1);
       close (pipefds[1]);
 
@@ -295,7 +296,6 @@ int client_launch_exec (int forkit, char *dir, char *argv[],
       on_sig (SIGALRM, _exit);
       on_sig (SIGCHLD, SIG_DFL);
 
-      sleep (15);
       _exit (1);
     }
     return pid;
@@ -374,6 +374,9 @@ struct condef *condef_new (struct server *s) {
 
 
 void condef_free (struct condef *con) {
+
+  g_return_if_fail(con!=NULL);
+
   if (con->s)
     server_unref (con->s);
 
