@@ -16,10 +16,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gnuconfig.h"
+
 #include <gtk/gtk.h>
+#include <dlfcn.h>
 
 #include "utils.h"
 #include "pixmaps.h"
+#include "game.h"
 
 
 #include "xpm/update.xpm"
@@ -36,31 +40,46 @@
 #include "xpm/pfilter.xpm"
 #include "xpm/pfilter-cfg.xpm"
 
+// hack to make dlsym work
+#define static
 #include "xpm/q.xpm"
 #include "xpm/q1.xpm"
 #include "xpm/q2.xpm"
 #include "xpm/q3.xpm"
 #include "xpm/wo.xpm"
+#include "xpm/et.xpm"
+#include "xpm/stvef.xpm"
 #include "xpm/hex.xpm"
 #include "xpm/hw.xpm"
 #include "xpm/sn.xpm"
 #include "xpm/hl.xpm"
 #include "xpm/kp.xpm"
 #include "xpm/sfs.xpm"
+#include "xpm/sof2s.xpm"
 #include "xpm/t2.xpm"
 #include "xpm/hr2.xpm"
 #include "xpm/un.xpm"
+#include "xpm/postal2.xpm"
+#include "xpm/aao.xpm"
 #include "xpm/rune.xpm"
 #include "xpm/descent3.xpm"
+#include "xpm/ut2.xpm"
+#include "xpm/ut2004.xpm"
 
 #include "xpm/gamespy3d.xpm"
+#include "xpm/ssam.xpm"
+#include "xpm/mohaa.xpm"
+#include "xpm/savage.xpm"
+#include "xpm/cod.xpm"
+#include "xpm/bf1942.xpm"
+#undef static
 
 #include "xpm/green-plus.xpm"
 #include "xpm/red-minus.xpm"
 
 #include "xpm/man-black.xpm"
 #include "xpm/man-red.xpm"
-#include "xpm/man-yellow.xpm"
+#include "xpm/man-yellow.xpm" // He's actually green
 
 #include "xpm/group-red.xpm"
 #include "xpm/group-green.xpm"
@@ -78,6 +97,8 @@
 #include "xpm/server-to.xpm"
 #include "xpm/server-error.xpm"
 #include "xpm/locked.xpm"
+#include "xpm/punkbuster.xpm"
+#include "xpm/locked_punkbuster.xpm"
 
 struct pixmap update_pix;
 struct pixmap refresh_pix;
@@ -88,35 +109,45 @@ struct pixmap connect_pix;
 struct pixmap observe_pix;
 struct pixmap record_pix;
 
-// Workaround for ppc compiler bug
-// which does not initialize arrays with 0
 struct pixmap filter_pix[2];
 struct pixmap filter_cfg_pix[2];
 
+#if 0
 struct pixmap q_pix;
 struct pixmap q1_pix;
 struct pixmap q2_pix;
 struct pixmap q3_pix;
 struct pixmap wo_pix;
+struct pixmap et_pix;
+struct pixmap ef_pix;
 struct pixmap hex_pix;
 struct pixmap hw_pix;
 struct pixmap sn_pix;
 struct pixmap hl_pix;
 struct pixmap kp_pix;
 struct pixmap sfs_pix;
+struct pixmap sof2s_pix;
 struct pixmap t2_pix;
 struct pixmap hr_pix;
 struct pixmap un_pix;
+struct pixmap postal2_pix;
+struct pixmap aao_pix;
+struct pixmap ut2_pix;
 struct pixmap rune_pix;
 struct pixmap descent3_pix;
 struct pixmap gamespy3d_pix;
+struct pixmap ssam_pix;
+struct pixmap mohaa_pix;
+struct pixmap savage_pix;
+struct pixmap cod_pix;
+#endif
 
 struct pixmap gplus_pix;
 struct pixmap rminus_pix;
 
 struct pixmap man_black_pix;
 struct pixmap man_red_pix;
-struct pixmap man_yellow_pix;
+struct pixmap man_yellow_pix; // He's actually green
 
 struct pixmap group_pix[3];
 struct pixmap buddy_pix[9];
@@ -125,6 +156,8 @@ struct pixmap error_pix;
 
 struct pixmap server_status[5];
 struct pixmap locked_pix;
+struct pixmap punkbuster_pix;
+struct pixmap locked_punkbuster_pix;
 
 static GdkGC *pixmaps_gc;
 static GdkGC *masks_gc;
@@ -153,7 +186,7 @@ int pixmap_width (GdkPixmap *pixmap) {
 }
 
 
-static void free_pixmap (struct pixmap *pixmap) {
+void free_pixmap (struct pixmap *pixmap) {
   if (!pixmap)
     return;
 
@@ -178,8 +211,9 @@ static void create_pixmap (GtkWidget *window, struct pixmap *pix,
 }
 
 
-void free_pixmaps (void) {
-  int i;
+void free_pixmaps (void)
+{
+  unsigned i;
 
   free_pixmap (&update_pix);
   free_pixmap (&refresh_pix);
@@ -195,30 +229,12 @@ void free_pixmaps (void) {
     free_pixmap (&filter_cfg_pix[i]);
   }
 
-  free_pixmap (&q_pix);
-  free_pixmap (&q1_pix);
-  free_pixmap (&q2_pix);
-  free_pixmap (&q3_pix);
-  free_pixmap (&wo_pix);
-  free_pixmap (&hex_pix);
-  free_pixmap (&hw_pix);
-  free_pixmap (&sn_pix);
-  free_pixmap (&hl_pix);
-  free_pixmap (&kp_pix);
-  free_pixmap (&sfs_pix);
-  free_pixmap (&t2_pix);
-  free_pixmap (&hr_pix);
-  free_pixmap (&un_pix);
-  free_pixmap (&rune_pix);
-  free_pixmap (&descent3_pix);
-  free_pixmap (&gamespy3d_pix);
-
   free_pixmap (&gplus_pix);
   free_pixmap (&rminus_pix);
 
   free_pixmap (&man_black_pix);
   free_pixmap (&man_red_pix);
-  free_pixmap (&man_yellow_pix);
+  free_pixmap (&man_yellow_pix); // He's actually green
 
   for (i = 0; i < 3; i++)
     free_pixmap (&group_pix[i]);
@@ -233,6 +249,16 @@ void free_pixmaps (void) {
 
   free_pixmap (&locked_pix);
 
+  free_pixmap (&punkbuster_pix);
+  free_pixmap (&locked_punkbuster_pix);
+
+  for (i = 0; i < GAMES_TOTAL; i++)
+  {
+    free_pixmap(games[i].pix);
+    g_free(games[i].pix);
+    games[i].pix = NULL;
+  }
+
   if (pixmaps_gc) {
     gdk_gc_destroy (pixmaps_gc);
     pixmaps_gc = NULL;
@@ -245,7 +271,10 @@ void free_pixmaps (void) {
 }
 
 
-void init_pixmaps (GtkWidget *window) {
+void init_pixmaps (GtkWidget *window)
+{
+  unsigned i = 0;
+
   free_pixmaps ();
 
   if (!GTK_WIDGET_REALIZED (window))
@@ -266,30 +295,12 @@ void init_pixmaps (GtkWidget *window) {
   create_pixmap (window, &filter_pix[1],     pfilter_xpm);
   create_pixmap (window, &filter_cfg_pix[1], pfilter_cfg_xpm);
 
-  create_pixmap (window, &q_pix, q_xpm);
-  create_pixmap (window, &q1_pix, q1_xpm);
-  create_pixmap (window, &q2_pix, q2_xpm);
-  create_pixmap (window, &q3_pix, q3_xpm);
-  create_pixmap (window, &wo_pix, wo_xpm);
-  create_pixmap (window, &hex_pix, hex_xpm);
-  create_pixmap (window, &hw_pix, hw_xpm);
-  create_pixmap (window, &sn_pix, sn_xpm);
-  create_pixmap (window, &hl_pix, hl_xpm);
-  create_pixmap (window, &kp_pix, kp_xpm);
-  create_pixmap (window, &sfs_pix, sfs_xpm);
-  create_pixmap (window, &t2_pix, t2_xpm);
-  create_pixmap (window, &hr_pix, hr2_xpm);
-  create_pixmap (window, &un_pix, un_xpm);
-  create_pixmap (window, &rune_pix, rune_xpm);
-  create_pixmap (window, &descent3_pix, descent3_xpm);
-  create_pixmap (window, &gamespy3d_pix, gamespy3d_xpm);
-
   create_pixmap (window, &gplus_pix, green_plus_xpm);
   create_pixmap (window, &rminus_pix, red_minus_xpm);
 
   create_pixmap (window, &man_black_pix, man_black_xpm);
   create_pixmap (window, &man_red_pix, man_red_xpm);
-  create_pixmap (window, &man_yellow_pix, man_yellow_xpm);
+  create_pixmap (window, &man_yellow_pix, man_yellow_xpm); // He's actually green
 
   create_pixmap (window, &group_pix[0], group_red_xpm);
   create_pixmap (window, &group_pix[1], group_green_xpm);
@@ -307,6 +318,32 @@ void init_pixmaps (GtkWidget *window) {
 
   create_pixmap (window, &error_pix, error_xpm);
   create_pixmap (window, &locked_pix, locked_xpm);
+  create_pixmap (window, &punkbuster_pix, punkbuster_xpm);
+  create_pixmap (window, &locked_punkbuster_pix, locked_punkbuster_xpm);
+
+  for (i = 0; i < GAMES_TOTAL; i++)
+  {
+    struct pixmap* pix = NULL;
+
+    pix = g_malloc(sizeof(struct pixmap));
+
+    if(games[i].icon)
+    {
+      char** xpm = dlsym(NULL, games[i].icon);
+      if(xpm)
+	create_pixmap (window, pix, xpm);
+    }
+
+    if(!pix)
+    {
+      pix->pix = error_pix.pix;
+      pix->mask = error_pix.mask;
+      gdk_pixmap_ref(pix->pix);
+      gdk_bitmap_ref(pix->mask);
+    }
+
+    games[i].pix = pix;
+  }
 }
 
 
