@@ -46,24 +46,6 @@ extern	struct pixmap record_pix;
 extern	struct pixmap filter_pix[];
 extern  struct pixmap filter_cfg_pix[];
 
-extern	struct pixmap q_pix;
-extern	struct pixmap q1_pix;
-extern	struct pixmap q2_pix;
-extern	struct pixmap q3_pix;
-extern	struct pixmap wo_pix;
-extern	struct pixmap hex_pix;
-extern	struct pixmap hw_pix;
-extern	struct pixmap sn_pix;
-extern	struct pixmap hl_pix;
-extern	struct pixmap kp_pix;
-extern	struct pixmap sfs_pix;
-extern	struct pixmap t2_pix;
-extern	struct pixmap hr_pix;
-extern	struct pixmap un_pix;
-extern	struct pixmap rune_pix;
-extern	struct pixmap descent3_pix;
-extern	struct pixmap gamespy3d_pix;
-
 extern	struct pixmap gplus_pix;
 extern	struct pixmap rminus_pix;
 
@@ -80,9 +62,13 @@ extern	struct pixmap server_status[];
 
 extern	struct pixmap locked_pix;
 
+extern	struct pixmap punkbuster_pix;
+extern	struct pixmap locked_punkbuster_pix;
+
 extern	int pixmap_height (GdkPixmap *pixmap);
 extern	int pixmap_width (GdkPixmap *pixmap);
 
+extern  void free_pixmap (struct pixmap *pixmap);
 extern	void free_pixmaps (void);
 extern	void init_pixmaps (GtkWidget *window);
 
