@@ -36,6 +36,7 @@ enum {
   PREF_PAGE_GAMES,
   PREF_PAGE_APPEARANCE,
   PREF_PAGE_QSTAT,
+  PREF_PAGE_SOUNDS
 };
 
 extern	char *user_rcdir;
@@ -75,7 +76,10 @@ extern  int 	default_nocdaudio;
 extern	int	show_hostnames;
 extern	int	show_default_port;
 
+extern	int	serverlist_countbots;
+
 extern	int	default_terminate;
+extern  int	default_stopxmms;
 extern	int	default_iconify;
 extern	int	default_launchinfo;
 extern	int	default_prelaunchexec;
@@ -83,31 +87,34 @@ extern  int 	default_save_lists;
 extern	int 	default_save_srvinfo;
 extern	int 	default_save_plrinfo;
 extern	int	default_auto_favorites;
+extern	int	default_show_splash;
 extern	int	default_always_resolve;
 extern	int	default_toolbar_style;
 extern	int	default_toolbar_tips;
 extern	int	default_refresh_sorts;
 extern	int	default_refresh_on_update;
+extern	int	default_resolve_on_update;
+extern	int	default_show_only_configured_games;
 
 extern	int	maxretries;
 extern	int	maxsimultaneous;
 
-/* Quake 3 settings */
-struct q3engineopts
-{
-  char *masterprotocol;
-  struct game *game;
-  int vmfix;
-  int rafix;
-  int setfs_game;
-};
+extern	int	sound_enable;
+extern	char	*sound_player;
+extern	char	*sound_xqf_start;
+extern	char	*sound_xqf_quit;
+extern	char	*sound_update_done;
+extern	char	*sound_refresh_done;
+extern	char	*sound_stop;
+extern	char	*sound_server_connect;
+extern	char	*sound_redial_success;
 
 // Quake 3 Arena
-extern struct q3engineopts q3_opts;
+//extern struct q3engineopts q3_opts;
 // Return to Castle Wolfenstein
-extern struct q3engineopts wo_opts;
+//extern struct q3engineopts wo_opts;
 // Any other game with Q3 engine
-extern struct q3engineopts generic_q3_opts;
+//extern struct q3engineopts generic_q3_opts;
 
 extern	int set_working_dir (char *dir);
 extern  void preferences_dialog (int page_num);
@@ -118,7 +125,6 @@ extern	void free_user_info (void);
 
 extern	int prefs_load (void);
 
-
 #endif /* __PREF_H__ */
 
 
