@@ -32,14 +32,18 @@
 #include "utils.h"
 #include "server.h"
 
+#ifdef USE_GEOIP
+#include "country-filter.h"
+#endif
+
 
 struct server_hash {
   int num;
   GSList **nodes;
 };
 
-static struct server_hash servers = { 251, NULL };
-static struct server_hash uservers = { 17, NULL };
+static struct server_hash servers = { 2333, NULL };
+static struct server_hash uservers = { 251, NULL };
 
 
 static int server_hash_func (const struct host *h, unsigned short port) {
@@ -91,7 +95,11 @@ static struct server *server_new (struct host *h, unsigned short port,
   server->type = type;
   server->ping = -1;
   server->retries = -1;
-
+	
+#ifdef USE_GEOIP
+  server->country_id = geoip_id_by_ip(h->ip);
+#endif
+		 	
   return server;
 }
 
@@ -160,7 +168,6 @@ struct server *server_add (struct host *h, unsigned short port,
   return server;
 }
 
-
 struct userver *userver_add (const char *hostname, unsigned short port,
                                                       enum server_type type) {
   GSList *ptr;
@@ -221,6 +228,7 @@ void server_free_info (struct server *s) {
   s->flags = 0;
   s->maxplayers = 0;
   s->curplayers = 0;
+  s->curbots = 0;
 
   s->filters = 0;
   s->flt_mask = 0;
@@ -228,11 +236,12 @@ void server_free_info (struct server *s) {
 }
 
 
-void server_unref (struct server *server) {
+// return NULL if refcount dropped to zero, server otherwise
+struct server* server_unref (struct server *server) {
   int node;
 
   if (!server)
-    return;
+    return NULL;
 
   server->ref_count--;
 
@@ -250,7 +259,9 @@ void server_unref (struct server *server) {
     host_unref (server->host);
     server_free_info (server);
     g_free (server);
+    return NULL;
   }
+  return server;
 }
 
 
@@ -274,6 +285,23 @@ void userver_unref (struct userver *s) {
   }
 }
 
+// change the server port to newport. this function is required as the server
+// needs to get a new position in the servers hash
+struct server* server_change_port (struct server* s, int newport)
+{
+    int node;
+
+    if(!newport || !s)
+	    return s;
+    
+    node = server_hash_func (s->host, s->port);
+    servers.nodes[node] = g_slist_remove (servers.nodes[node], s);
+    s->port = newport;
+    node = server_hash_func (s->host, s->port);
+    // FIXME: there could be already a server with that port
+    servers.nodes[node] = g_slist_prepend (servers.nodes[node], s);
+    return s;
+}
 
 GSList *server_list_copy (GSList *list) {
   debug (3, "server_list_copy() -- list %ld copying all servers", list);
@@ -330,13 +358,21 @@ GSList *userver_list_append_list (GSList *list, GSList *uservers,
 
 int servers_total (void) {
   int i;
-  int size = 0;
+  int size = 0, len;
+  int min=-1, max=0;
 
   if (!servers.nodes)
     return 0;
 
   for (i = 0; i < servers.num; i++)
-    size += g_slist_length (servers.nodes[i]);
+  {
+    len = g_slist_length (servers.nodes[i]);
+    size += len;
+    if (min == -1 || len < min) min=len;
+    if (len > max) max=len;
+  }
+
+  debug(1,"server hash (min/max/avg) %d/%d/%5.2f",min,max,size/(servers.num*1.0));
 
   return size;
 }
@@ -528,5 +564,3 @@ void userver_list_fprintf (FILE *f, GSList *uservers) {
     uservers = uservers->next;
   }
 }
-
-
