@@ -19,6 +19,8 @@
 #ifndef __SERVER_H__
 #define __SERVER_H__
 
+#include <stdio.h>
+
 #include <glib.h>
 
 #include "xqf.h"
@@ -30,9 +32,12 @@ extern	struct userver *userver_add (const char *name, unsigned short port,
                                                        enum server_type type);
 extern	void server_free_info (struct server *s);
 
-extern	void server_unref (struct server *s);
+// return NULL if refcount dropped to zero, s otherwise
+extern	struct server* server_unref (struct server *s);
 extern	void userver_unref (struct userver *s);
 
+struct server* server_change_port (struct server* s, int newport);
+
 extern	GSList *server_list_copy (GSList *list);
 extern	GSList *userver_list_copy (GSList *list);
 
@@ -56,7 +61,6 @@ extern	void server_lists_intersect (GSList **list1, GSList **list2);
 extern	void server_list_fprintf (FILE *f, GSList *servers);
 extern	void userver_list_fprintf (FILE *f, GSList *uservers);
 
-
 static inline void server_ref (struct server *server) {
   if (server){ 
     server->ref_count++;
@@ -77,6 +81,15 @@ static inline GSList *server_list_prepend (GSList *list, struct server *server)
   return list;
 }
 
+// This version does not to the duplicate check - you must do it yourself
+static inline GSList *server_list_prepend_ndp (GSList *list, struct server *server) {
+  debug (6, "server_list_prepend_ndp() -- Server %lx", server);
+  list = g_slist_prepend (list, server);
+  server_ref (server);
+
+  return list;
+}
+
 static inline GSList *server_list_append (GSList *list, struct server *server) {
   if (g_slist_find (list, server) == NULL) {
     debug (6, "server_list_append() -- Server %lx", server);
