@@ -16,6 +16,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gnuconfig.h"
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>	/* strtol */
@@ -27,12 +29,9 @@
 #include <arpa/inet.h>	/* inet_aton, inet_ntoa */
 #include <errno.h>	/* errno */
 
-#ifndef errno
-extern	int errno;
-#endif
-
 #include <gtk/gtk.h>
 
+#include "i18n.h"
 #include "xqf.h"
 #include "xqf-ui.h"
 #include "source.h"
@@ -47,17 +46,25 @@ extern	int errno;
 #include "zipped.h"
 #include "stat.h"
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(string) gettext(string)
-#else
-#define _(string) (string)
-#endif
-
-
 struct master *favorites = NULL;
 GSList *master_groups = NULL;
 
+char* master_prefixes[MASTER_NUM_QUERY_TYPES] = {
+	"master://",
+	"gmaster://",
+	"http://",
+	"lan://",
+	"file://"
+};
+
+char* master_designation[MASTER_NUM_QUERY_TYPES] = {
+	N_("Standard"),
+	N_("Gamespy"),
+	N_("http"),
+	N_("LAN"),
+	N_("File")
+};
+
 static GSList *all_masters = NULL;
 
 
@@ -77,20 +84,37 @@ static void save_list (FILE *f, struct master *m) {
     fprintf (f, "[%s]\n", m->name);
   }
   else {
+    /*
     if (m->url) {
       fprintf (f, "[%s %s]\n", games[m->type].id, m->url);
     }
-    else {
-      if (m->master_type == 1)
-	{
-	fprintf (f, "[%s %s%s:%d]\n", games[m->type].id, PREFIX_GMASTER,
+    else if (m->master_type == MASTER_GAMESPY)
+    {
+      fprintf (f, "[%s %s%s:%d]\n", games[m->type].id, master_prefixes[MASTER_GAMESPY],
         	(m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
-        }
-      else
-        {
-      	fprintf (f, "[%s %s%s:%d]\n", games[m->type].id, PREFIX_MASTER,
+    }
+    else
+    {
+      fprintf (f, "[%s %s%s:%d]\n", games[m->type].id, master_prefixes[MASTER_NATIVE],
 		(m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
-	}
+    }
+*/
+    
+    switch(m->master_type)
+    {
+      case MASTER_NATIVE:
+      case MASTER_GAMESPY:
+      case MASTER_LAN:
+	fprintf (f, "[%s %s%s:%d]\n", games[m->type].id, master_prefixes[m->master_type],
+        	(m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
+	break;
+      case MASTER_HTTP:
+      case MASTER_FILE:
+	fprintf (f, "[%s %s]\n", games[m->type].id, m->url);
+	break;
+      case MASTER_NUM_QUERY_TYPES:
+      case MASTER_INVALID_TYPE:
+	break;
     }
   }
 
@@ -177,8 +201,8 @@ static void save_server_info (const char *filename, GSList *servers) {
   if (z.f) {
     while (servers) {
       s = (struct server *) servers->data;
-
-      if (games[s->type].save_info)
+	
+      if (s->ref_count > 0 && games[s->type].save_info)
 	(*games[s->type].save_info) (z.f, s);
 
       servers = servers->next;
@@ -330,35 +354,46 @@ static struct master *find_master_url (char *url) {
 static struct master *read_list_parse_master (char *str, char *str2) {
   char *addr;
   unsigned short port;
-  struct master *m;
+  struct master *m = NULL;
+  enum master_query_type query_type;
 
   if (favorites && !g_strcasecmp (str, favorites->name))
     return favorites;
 
-  if (g_strncasecmp (str, PREFIX_MASTER, sizeof (PREFIX_MASTER) - 1) == 0) {
-    if (parse_address (str + sizeof (PREFIX_MASTER) - 1, &addr, &port)) {
-      m = find_master_server (addr, port, str2);
-      g_free (addr);
-      return m;
-    }
+  query_type = get_master_query_type_from_address(str);
+
+  switch(query_type)
+  {
+    case MASTER_NATIVE:
+    case MASTER_GAMESPY:
+    case MASTER_LAN:
+      if (parse_address (str + strlen(master_prefixes[query_type]), &addr, &port))
+      {
+	m = find_master_server (addr, port, str2);
+	g_free (addr);
+      }
+      break;
+    case MASTER_HTTP:
+    case MASTER_FILE:
+      m = find_master_url (str);
+      break;
+    case MASTER_NUM_QUERY_TYPES:
+    case MASTER_INVALID_TYPE:
+      break;
   }
 
-  if (g_strncasecmp (str, PREFIX_GMASTER, sizeof (PREFIX_GMASTER) - 1) == 0) {
-    if (parse_address (str + sizeof (PREFIX_GMASTER) - 1, &addr, &port)) {
-      m = find_master_server (addr, port, str2);
-      g_free (addr);
-      return m;
-    }
-  }
-
-  if (g_strncasecmp (str, PREFIX_URL_HTTP, sizeof (PREFIX_URL_HTTP) - 1) == 0) {
-    m = find_master_url (str);
-    return m;
-  }
-
-  return NULL;
+  return m;
 }
 
+static gint server_sorting_helper (const struct server *s1, 
+				      const struct server *s2) {
+  if(s1==s2)
+    return 0;
+  else if(s1<s2)
+    return -1;
+  else
+    return 1;
+}
 
 static void master_add_server (struct master *m, char *str, 
                                                       enum server_type type) {
@@ -374,9 +409,9 @@ static void master_add_server (struct master *m, char *str,
     if (h) {						/* IP address */
       host_ref (h);
       if ((s = server_add (h, port, type)) != NULL) {
-	m->servers = server_list_prepend (m->servers, s);
+	m->servers = server_list_prepend_ndp (m->servers, s);
 	/* Since the server_add increments the ref count, and 
-	   server_list_prepend ups the ref_count, we should
+	   server_list_prepend_ndp ups the ref_count, we should
 	   unref it once because we are only keeping it in
 	   one list after this function.
 	*/
@@ -393,7 +428,6 @@ static void master_add_server (struct master *m, char *str,
   }
 }
 
-
 static void read_lists (const char *filename) {
   struct zstream z;
   char *realname;
@@ -450,7 +484,7 @@ static void read_lists (const char *filename) {
 	master_add_server (m, token[1], type);
     }
   }
-
+  
   if (m && m->servers)
     m->servers = g_slist_reverse (m->servers);
 
@@ -466,6 +500,8 @@ static void read_server_info (const char *filename) {
   char *pos;
   GSList *strings;
 
+  debug(3,"read_server_info(%s)",filename);
+
   realname = file_in_dir (user_rcdir, filename);
   zstream_open_r (&z, realname);
 
@@ -592,14 +628,147 @@ static struct master *create_master (char *name, enum server_type type,
 
 struct master *add_master (char *path, char *name, enum server_type type, 
                                                   int user, int lookup_only) {
-  char *addr;
-  unsigned short port;
+  char *addr = NULL;
+  unsigned short port = 0;
   struct master *m = NULL;
-  struct host *h;
-  struct master *group;
+  struct host *h = NULL;
+  struct master *group = NULL;
+  enum master_query_type query_type;
 
-  if (g_strncasecmp (path, PREFIX_MASTER, sizeof (PREFIX_MASTER) - 1) == 0) {
-    if (parse_address (path + sizeof (PREFIX_MASTER) - 1, &addr, &port)) {
+  debug(6,"add_master(%s,%s,%d,%d,%d)",path,name,type,user,lookup_only);
+
+  query_type = get_master_query_type_from_address(path);
+  if( query_type == MASTER_INVALID_TYPE )
+  {
+    debug(1,"Invalid Master %s",path);
+    return NULL;
+  }
+
+  switch(query_type)
+  {
+    case MASTER_NATIVE:
+    case MASTER_GAMESPY:
+    case MASTER_LAN:
+      // check for valid hostname/ip
+      if (parse_address (path + strlen(master_prefixes[query_type]), &addr, &port))
+      {
+	// if no port was specified, add default master port if available or fail
+	if (!port)
+	{
+	  // use default_port instead of default_master_port for lan broadcasts
+	  if( query_type == MASTER_LAN )
+	  {
+	    port = games[type].default_port;
+	    /* no longer necessary as of qstat 2.5c
+	    // unreal needs one port higher
+	    if(!strcmp(games[type].qstat_option,"-uns"))
+	    {
+	      port++;
+	      type=GPS_SERVER;
+	    }
+	    */
+	  }
+	  // do not use default for gamespy
+	  else if (query_type != MASTER_GAMESPY && games[type].default_master_port)
+	  {
+	    port = games[type].default_master_port;
+	  }
+	  else
+	  {
+	    g_free (addr);
+	    // translator: %s == url, eg gmaster://bla.blub.org
+	    dialog_ok (NULL, _("You have to specify a port number for %s."),path);
+	    return NULL;
+	  }
+	}
+
+	m = find_master_server (addr, port, games[type].id);
+
+      }
+      break;
+
+    case MASTER_HTTP:
+    case MASTER_FILE:
+      m = find_master_url (path);
+      break;
+    case MASTER_NUM_QUERY_TYPES:
+    case MASTER_INVALID_TYPE:
+      return NULL;
+  }
+
+  if (lookup_only)
+  {
+    g_free (addr);
+    return m;
+  }
+
+  if (m)
+  {
+    if (user)
+    { // Master renaming is forced by user
+      g_free (m->name);
+      m->name = g_strdup (name);
+      m->user = TRUE;
+    }
+    else
+    { // Automatically rename masters that are not edited by user
+      if (!m->user) {
+	g_free (m->name);
+	m->name = g_strdup (name);
+      }
+    }
+    g_free (addr);
+    return m;
+  }
+  
+  // master was not known already, create new
+  switch(query_type)
+  {
+    case MASTER_NATIVE:
+    case MASTER_GAMESPY:
+    case MASTER_LAN:
+      m = create_master (name, type, FALSE);
+
+      h = host_add (addr);
+      if (h) {
+	m->host = h;
+	host_ref (h);
+	g_free (addr);
+	addr = NULL;
+      }
+      else {
+	m->hostname = addr;
+      }
+      m->port = port;
+      break;
+    case MASTER_HTTP:
+    case MASTER_FILE:
+      m = create_master (name, type, FALSE);
+      m->url = g_strdup (path);
+      break;
+    case MASTER_NUM_QUERY_TYPES:
+    case MASTER_INVALID_TYPE:
+      return NULL;
+  }
+  
+  m->master_type = query_type;
+
+  if (m) {
+    group = (struct master *) g_slist_nth_data (master_groups, type);
+    group->masters = g_slist_append (group->masters, m);
+    m->user = user;
+  }
+  
+  return m;
+}
+
+
+
+#if 0
+{ /* for xemacs so it does not loose its mind -- baa */
+  if (g_strncasecmp (path, master_prefixes[MASTER_NATIVE],
+			  strlen(master_prefixes[MASTER_NATIVE])) == 0) {
+    if (parse_address (path + strlen(master_prefixes[MASTER_NATIVE]), &addr, &port)) {
 
       if (!port) {
 	if (games[type].default_master_port) {
@@ -639,7 +808,7 @@ struct master *add_master (char *path, char *name, enum server_type type,
        {
 	m = create_master (name, type, FALSE);
 
-        m->master_type = 0; // Regular master
+        m->master_type = MASTER_NATIVE; // Regular master
 
 	h = host_add (addr);
 	if (h) {
@@ -656,8 +825,9 @@ struct master *add_master (char *path, char *name, enum server_type type,
     }
   }
 
-  else if (g_strncasecmp (path, PREFIX_GMASTER, sizeof (PREFIX_GMASTER) - 1) == 0) {
-    if (parse_address (path + sizeof (PREFIX_GMASTER) - 1, &addr, &port)) {
+  else if (g_strncasecmp (path, master_prefixes[MASTER_GAMESPY],
+			  strlen(master_prefixes[MASTER_GAMESPY])) == 0) {
+    if (parse_address (path + strlen(master_prefixes[MASTER_GAMESPY]), &addr, &port)) {
 
       if (!port) {
 	if (games[type].default_master_port) {
@@ -697,7 +867,7 @@ struct master *add_master (char *path, char *name, enum server_type type,
       {
 	m = create_master (name, type, FALSE);
 
-        m->master_type = 1; // Gamespy master
+        m->master_type = MASTER_GAMESPY; // Gamespy master
 
 	h = host_add (addr);
 	if (h) {
@@ -715,8 +885,8 @@ struct master *add_master (char *path, char *name, enum server_type type,
   }
 
   else {
-    if (g_strncasecmp (path, PREFIX_URL_HTTP, sizeof (PREFIX_URL_HTTP) - 1) 
-                                                                       == 0) {
+    if (g_strncasecmp (path, master_prefixes[MASTER_HTTP],
+			    strlen(master_prefixes[MASTER_HTTP])) == 0) {
       m = find_master_url (path);
 
       if (lookup_only) {
@@ -745,7 +915,6 @@ struct master *add_master (char *path, char *name, enum server_type type,
       }
     }
   }
-
   if (m) {
     group = (struct master *) g_slist_nth_data (master_groups, type);
     group->masters = g_slist_append (group->masters, m);
@@ -754,6 +923,7 @@ struct master *add_master (char *path, char *name, enum server_type type,
 
   return m;
 }
+#endif
 
 
 void free_master (struct master *m) {
@@ -796,15 +966,8 @@ static char *builtin_masters_update_info[] = {
   "ADD QWS master://192.246.40.37:27003 id Team Fortress",
   "ADD QWS master://192.246.40.37:27004 id Misc",
   "ADD QWS master://192.246.40.37:27006 id Deathmatch",
-  "ADD QWS master://204.182.161.2:27000 PlanetQuake",
-  "ADD QWS master://203.55.240.100:27000 Australia",
-  "ADD QWS master://200.255.244.2:27000 Brazil",
   "ADD QWS master://qwmaster.ocrana.de:27000 Germany",
-  "ADD QWS master://207.88.6.18:27000 FortressFest TF",
-  "ADD QWS master://master.quakeworld.ru:27000 Russia",
-  "ADD QWS master://194.251.249.32:27000 iNET (Finland)",
-  "ADD QWS master://qwmaster.mondial.net.au:27000 Mondial Aussie",
-  "ADD QWS master://qwmaster.barrysworld.com:27000 BarrysWorld (UK)",
+  "ADD QWS master://santa.quakeforge.net:27000 QuakeForge",
 
   "ADD Q2S master://satan.idsoftware.com:27900 id",
   "ADD Q2S master://q2master.planetquake.com:27900 PlanetQuake", 
@@ -816,50 +979,90 @@ static char *builtin_masters_update_info[] = {
   "ADD Q2S http://www.gameaholic.com/servers/qspy-quake2 Gameaholic.Com",
   "ADD Q2S http://www.lithium.com/quake2/gamespy.txt Lithium",
 
-#ifdef QSTAT23
   "ADD Q3S master://master3.idsoftware.com id",
-  "ADD Q3S master://q3master.splatterworld.de Germany",
-  "ADD Q3S master://q3.golsyd.net.au Australia",
+//  "ADD Q3S master://q3master.splatterworld.de Germany",
+//  "ADD Q3S master://q3.golsyd.net.au Australia",
   "ADD Q3S master://q3master.barrysworld.com:27950 BarrysWorld",
-#endif
+  "ADD Q3S http://www.gameaholic.com/servers/qspy-quake3 Gameaholic.com",
+
+//  "ADD HWS master://santa.quakeforge.net:26900 QuakeForge",
 
   "ADD SNS http://www.gameaholic.com/servers/qspy-sin Gameaholic.Com",
   "ADD SNS http://asp.planetquake.com/sinserverlist/servers.txt PlanetQuake",
 
-#ifdef QSTAT23
   "ADD HLS master://half-life.east.won.net WON East",
   "ADD HLS master://half-life.west.won.net WON West",
-#endif
 
   "ADD Q2S:KP http://www.gameaholic.com/servers/qspy-kingpin Gameaholic.Com",
   "ADD Q2S:KP http://www.ogn.org:6666 OGN",
 
   "ADD Q2S:HR http://www.gameaholic.com/servers/qspy-heretic2 Gameaholic.Com",
 
-#ifdef QSTAT_HAS_UNREAL_SUPPORT
   "ADD UNS gmaster://unreal.epicgames.com:28900 Epic",
   "ADD UNS gmaster://utmaster.barrysworld.com:28909 BarrysWorld",
-#endif
 
-  "ADD T2S master://198.74.33.29:28002 NA West1",
-  "ADD T2S master://64.94.105.141:27999 NA West2",
-  "ADD T2S master://64.94.105.145:28000 NA West3",
-  "ADD T2S master://209.67.28.144:28002 NA East1",
-  "ADD T2S master://209.67.28..145:27999 NA East2",
+  "DELETE T2S master://211.233.32.77:28002 Tribes2 Master", // does no longer work
+
+  "ADD T2S master://198.74.32.54:27999 Master 1",
+  "ADD T2S master://198.74.32.55:27999 Master 2",
+  "ADD T2S master://211.233.86.203:28002 Master 3",
 
   "ADD WOS master://wolf.idsoftware.com:27950 id",
+  "ADD WOETS master://etmaster.idsoftware.com:27950 id",
+
+  "ADD EFS http://www.gameaholic.com/servers/qspy-startrekeliteforce Gameaholic.com",
+  "ADD EFS master://master.stef1.ravensoft.com:27953  Ravensoft",
 
   "ADD D3P master://gt.pxo.net:3445 PXO",
   "ADD D3P http://www.gameaholic.com/servers/qspy-descent3 gameaholic.com",
 
   "ADD SFS http://www.gameaholic.com/servers/qspy-soldieroffortune gameaholic.com",
-  "ADD GPS http://www.gameaholic.com/servers/qspy-noonelivesforever Noone Lives Forevere - gameaholic.com",
-  "ADD GPS http://www.gameaholic.com/servers/qspy-rallymasters Rally Masters - gameaholic.com",
-  "ADD GPS http://www.gameaholic.com/servers/qspy-turok2 Turok 2 - gameaholic.com",
-  "ADD GPS http://www.gameaholic.com/servers/qspy-shogo Shogo - gameaholic.com",
 
   "ADD RUNESRV http://www.gameaholic.com/servers/qspy-rune gameaholic.com",
 
+  "ADD SOF2S master://master.sof2.ravensoft.com:20110 Raven",
+
+  "ADD UT2S http://ut2003master.epicgames.com/serverlist/full-all.txt Epic",
+  "ADD UT2S http://ut2003master.epicgames.com/serverlist/demo-all.txt Epic Demo",
+  
+  "ADD SAS http://masterserver.savagedemo.s2games.com/gamelist.dat S2 Games",
+
+  "ADD UNS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=UT&hostport=1 multiplay.co.uk - UT",
+  "ADD MHS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=MOHAA&hostport=1 multiplay.co.uk - MOHAA",
+  "ADD EFS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=EF&hostport=1 multiplay.co.uk - Elite Force",
+
+  // version 2.0 not listed here
+  "DELETE AMS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=AA&hostport=1 multiplay.co.uk - Army Ops",
+  "ADD SOF2S http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=SOF2&hostport=1 multiplay.co.uk - SOF2",
+  "ADD UT2S http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=UT2K3&hostport=1 multiplay.co.uk - UT2003",
+  "ADD UT2004S http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=UT2K4&hostport=1 multiplay.co.uk - UT2004",
+  "ADD UT2004S http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=UT2K4D&hostport=1 multiplay.co.uk - UT2004 Demo",
+  "ADD HLS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=HLS&hostport=1 multiplay.co.uk - Half-Life",
+
+  "ADD CODS master://cod01.activision.com Activision",
+  "ADD CODS http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=COD multiplay.co.uk - Call of Duty",
+
+  "ADD BF1942 http://tourneys.multiplay.co.uk/public/servers.pl?opt=ListGamespy&event=Online&type=BF1942&hostport=1 multiplay.co.uk - BF1942",
+
+  "ADD QS lan://255.255.255.255 LAN",
+  "ADD QWS lan://255.255.255.255 LAN",
+  "ADD Q2S lan://255.255.255.255 LAN",
+  "ADD HLS lan://255.255.255.255 LAN",
+  "ADD Q3S lan://255.255.255.255 LAN",
+  "ADD WOS lan://255.255.255.255 LAN",
+  "ADD WOETS lan://255.255.255.255 LAN",
+  "ADD EFS lan://255.255.255.255 LAN",
+  "ADD UT2S lan://255.255.255.255 LAN",
+  "ADD UT2004S lan://255.255.255.255 LAN",
+  "ADD UNS lan://255.255.255.255 LAN",
+  "ADD RUNESRV lan://255.255.255.255 LAN",
+  "ADD MHS lan://255.255.255.255 LAN",
+  "ADD AMS lan://255.255.255.255 LAN",
+  "ADD CODS lan://255.255.255.255 LAN",
+  "ADD T2S lan://255.255.255.255 LAN",
+  "ADD POSTAL2 lan://255.255.255.255 LAN",
+  "ADD SFS lan://255.255.255.255 LAN",
+  
   NULL
 };
 
@@ -914,6 +1117,49 @@ void update_master_list_builtin (void) {
     update_master_list_action (*ptr);
 }
 
+// Refresh list of games and masters in left pane
+void refresh_source_list (void) {
+  GtkCTreeNode *node;
+  GSList *list;
+  struct master *m;
+  struct master *group = NULL;
+
+  // Re-add masters to the list, or remove them if game is not
+  // configured and set to show only configured games.
+  for (list = all_masters; list; list = list->next) {
+    m = (struct master *) list->data;
+
+    if (m == favorites || m->isgroup)
+      continue;
+
+    // Look for existing entry in ctree so we only re-add if needed,
+    // otherwise everthing gets re-expanded
+    node = gtk_ctree_find_by_row_data (GTK_CTREE (source_ctree), NULL, m);
+
+    // If it's not there check to see if we should add it
+    if (!node) // It's not in the list so maybe we should add it
+    {
+      if (!default_show_only_configured_games || games[m->type].cmd)
+        source_ctree_add_master (source_ctree, m);
+    }
+    else // It's in the list so maybe we should delete it
+      if (default_show_only_configured_games && !games[m->type].cmd)
+        source_ctree_delete_master (source_ctree, m);
+  }
+  // Remove master group if set to show only configured games.
+  for (list = all_masters; list; list = list->next) {
+    m = (struct master *) list->data;
+
+    if (m == favorites)
+      continue;
+
+    if (default_show_only_configured_games && !games[m->type].cmd) {
+      group = (struct master *) g_slist_nth_data (master_groups, m->type);
+      source_ctree_remove_master_group (source_ctree, group);
+    }
+  }
+}
+
 void update_master_list_web (void) {
   char **ptr;
 
@@ -1008,12 +1254,7 @@ static void save_master_list (void) {
       confstr = g_strjoin (" ", typeid, m->url, m->name, NULL);
     }
     else {
-
-      if (m->master_type == 1)
-	      addr = g_strdup_printf (PREFIX_GMASTER "%s:%d", 
-        	       (m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
-      else
-      	      addr = g_strdup_printf (PREFIX_MASTER "%s:%d", 
+      addr = g_strdup_printf ("%s%s:%d", master_prefixes[m->master_type],
         	       (m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
 
       confstr = g_strjoin (" ", typeid, addr, m->name, NULL);
@@ -1028,12 +1269,14 @@ static void save_master_list (void) {
   config_pop_prefix ();
 }
 
-
 void init_masters (int update) {
+  typedef void (*server_unref_void)(void*);
   struct master *m;
+  struct master *m2;
   int i;
+  GSList *list;
 
-  favorites = create_master ("Favorites", UNKNOWN_SERVER, FALSE);
+  favorites = create_master (N_("Favorites"), UNKNOWN_SERVER, FALSE);
 
   for (i = 0; i < GAMES_TOTAL; i++) {
     m = create_master (games[i].name, i, TRUE);
@@ -1050,7 +1293,27 @@ void init_masters (int update) {
   compat_convert_favorites ();
 
   read_lists (FILENAME_FAVORITES);
+  debug (2, "starting to read server list");
   read_lists (FILENAME_LISTS);
+  debug (2, "finished reading server list");
+
+  // Go through all servers, sort them and then remove duplicates
+  // master_add_server now uses server_list_prepend_ndp which does not
+  // search for duplicates while adding servers.  This is much faster.
+  // Because of this, duplicate checking must be done here.
+  // Only working on servers, not uservers.  Lists file only contains IPs.
+  debug (2, "Searching for duplicate server entries");
+
+  // for each master
+  for (list = all_masters; list; list = list->next) {
+    m2 = (struct master *) list->data;
+    if(!m2)
+      continue;
+    debug (2, "  Working on master: %s",m2->name);
+
+    m2->servers = slist_sort_remove_dups(m2->servers,(GCompareFunc)server_sorting_helper,(server_unref_void)server_unref);
+    
+  }
 
   read_server_info (FILENAME_SRVINFO);
 }
@@ -1207,3 +1470,29 @@ GSList *references_to_server (struct server *s) {
   return res;
 }
 
+enum master_query_type get_master_query_type_from_address(char* address)
+{
+  enum master_query_type type;
+  // check for known master prefix
+  debug(6,"get_master_query_type_from_address(%s)",address);
+  for (type=MASTER_NATIVE;type<MASTER_NUM_QUERY_TYPES;type++)
+  {
+    if(!g_strncasecmp( address, master_prefixes[type],
+	strlen(master_prefixes[type])))
+    {
+      debug(6,"get_master_query_type_from_address: found %s",master_prefixes[type]);
+      return type;
+    }
+  }
+  // only accept if there is no :// part at all
+  if(lowcasestrstr(address,"://"))
+  {
+    debug(6,"get_master_query_type_from_address: invalid");
+    return MASTER_INVALID_TYPE;
+  }
+  else
+  {
+    debug(6,"get_master_query_type_from_address: default native");
+    return MASTER_NATIVE;
+  }
+}
