@@ -20,13 +20,12 @@
 #include <stdlib.h>	/* strtoul */
 #include <string.h>	/* strcmp */
 
-#include <gtk/gtk.h>
-
-#include "xqf.h"
+#include "xqf-ui.h"
 #include "sort.h"
 #include "pixmaps.h"
 #include "srv-info.h"
 
+// TODO: put into external file
 
 #define	QW_TEAMPLAY_CTF 	11
 
@@ -335,6 +334,120 @@ static char *q3_dmflags[Q3_DMFLAGS] = {
   "Fixed FOV"		 	/*  32768 */
 };
 
+#define Q3_GENERATIONS_DMFLAGS	16
+
+static char *q3_generations_dmflags[Q3_GENERATIONS_DMFLAGS] = {
+  "No health",			// 1
+  "No powerups",		// 2
+  "No armor",			// 4
+  "No falling damage",		// 8
+  
+  "Fully Loaded",		// 16
+  "Infinite Ammo",		// 32
+  "Quad Drop",			// 64
+  "Spawn Farthest",		// 128
+  
+  "Force Respawn",		// 256
+  "Fixed FOV",			// 512
+  "No spectator",		// 1024
+  "No CTF techs",		// 2048
+  
+  "Allow Grapple to sky",	// 4096
+  "Weapon Grapple",		// 8192
+  "Offhand Grapple",		// 16384
+  "No Footsteps Sound",		// 32768
+};
+
+#define Q3_GENERATIONS_GENFLAGS	7
+
+static char *q3_generations_genflags[Q3_GENERATIONS_GENFLAGS] = {
+  "",
+  "Disable Earth",		// 2 You cannot select the Earth Soldiers class.
+  "Disable Doom",		// 4 You cannot select the Doom Warriors class.
+  "Disable Slipgate",		// 8 You cannot select the Slipgaters class.
+  "Disable Strogg",		// 16 You cannot select the Strogg Troopers class.
+
+  "Disable Arena",		// 32 You cannot select the Arena Gladiators class.
+  "Class-Based Teams ",		// 64 In DM, each class is its own team.
+};
+
+#define Q3_Q3UT3_VOTEFLAGS	6
+static char *q3_q3ut3_voteflags[Q3_Q3UT3_VOTEFLAGS] = {
+  "Map, Team, Friendlyfire",    // 1
+  "Gametype, Wave Respawn",	// 2
+  "Time-, Capture-, Fraglimit",	// 4
+  "Various delays and times",	// 8
+  "Matchmode, exec",		// 16
+  "Gear",			// 32
+};
+
+#define Q3_Q3UT3_GEARFLAGS	6
+static char *q3_q3ut3_gearflags[Q3_Q3UT3_GEARFLAGS] = {
+    "No Grenades",			// 1
+    "No Snipers (psg1 or sr8)",		// 2
+    "No Spas",				// 4
+    "No Pistols",			// 8
+    "No Autos (primary or secondary)",  // 16
+    "No Negev",				// 32
+};
+
+#define Q3_FREEZE_DMFLAGS	11
+
+static char *q3_freeze_dmflags[Q3_FREEZE_DMFLAGS] = {
+  NULL, 	                /*      1 */
+  NULL,				/*      2 */
+  NULL,				/*      4 */
+  "No Falling Damage",		/*      8 */
+				            
+  "Fixed FOV",			/*     16 */
+  "No footsteps",		/*     32 */
+  "No item reset",		/*     64 */
+  "No team reset",		/*    128 */
+				            
+  "Weapons stay",		/*    256 */
+  "No playerclip",		/*    512 */
+  "Nightmare mode",		/*   1024 */
+};
+
+#define RTCW_VOTEFLAGS	8
+
+static char *rtcw_voteflags[RTCW_VOTEFLAGS] = {
+  "Restart Map",	/*      1 */
+  "Reset Match",	/*      2 */
+  "Start Match",	/*      4 */
+  "Next Map",		/*      8 */
+				            
+  "Swap Teams",		/*     16 */
+  "Game Type",		/*     32 */
+  "Kick Player",	/*     64 */
+  "Change Map",		/*    128 */
+};
+
+#define WOET_VOTEFLAGS	17
+
+static char *woet_voteflags[WOET_VOTEFLAGS] = {
+  "No Competition Settings",	/*      1 */
+  "No Game Type",		/*      2 */
+  "No Player Kick",		/*      4 */
+  "No Map Change",		/*      8 */
+				            
+  "No Match Reset",		/*     16 */
+  "No Mute Spectators",		/*     32 */
+  "No Next Map",		/*     64 */
+  "No Public Settings",		/*    128 */
+  
+  "No Referee",			/*    256 */
+  "No Shuffle Teams by XP",	/*    512 */
+  "No Swap Teams",		/*   1024 */
+  "No Friendly Fire",		/*   2048 */
+  
+  "No Timelimit",		/*   4096 */
+  "No Warmup Damage",		/*   8192 */
+  "No Anti-Lag",		/*  16384 */
+  "No Balanced Teams",		/*  32768 */
+  
+  "No Muting",			/*  65536 */
+};
 
 static void show_extended_flags (const char *str, char *names[], int size,
                                           int showall, GtkCTreeNode *parent) {
@@ -374,6 +487,7 @@ static void show_extended_flags (const char *str, char *names[], int size,
 }
 
 
+// TODO: get rid of switch, put game specific functions into game struct
 void srvinf_ctree_set_server (struct server *s) {
   char *text[2];
   char **info;
@@ -470,13 +584,50 @@ void srvinf_ctree_set_server (struct server *s) {
       }
       break;
 
-#ifdef QSTAT23
     case Q3_SERVER:
       if (info[0] && !strcmp (info[0], "dmflags")) {
-	show_extended_flags (info[1], q3_dmflags, Q3_DMFLAGS, FALSE, node);
+	if (s->game && !g_strcasecmp (s->game, "generations")) {
+	  show_extended_flags (info[1], q3_generations_dmflags, Q3_GENERATIONS_DMFLAGS, FALSE, node);
+	}
+	else if (s->game && !g_strcasecmp (s->game, "freeze")) {
+	  show_extended_flags (info[1], q3_freeze_dmflags, Q3_FREEZE_DMFLAGS, FALSE, node);
+	}
+	else
+	{
+	  show_extended_flags (info[1], q3_dmflags, Q3_DMFLAGS, FALSE, node);
+	}
+      }
+      else if (info[0] && !g_strcasecmp (info[0], "g_allowvote")) {
+	if (s->game && !g_strcasecmp (s->game, "q3ut3")) {
+	  show_extended_flags (info[1], q3_q3ut3_voteflags, Q3_Q3UT3_VOTEFLAGS, FALSE, node);
+	}
+      }
+      else if (info[0] && !g_strcasecmp (info[0], "g_gear")) {
+	if (s->game && !g_strcasecmp (s->game, "q3ut3")) {
+	  show_extended_flags (info[1], q3_q3ut3_gearflags, Q3_Q3UT3_GEARFLAGS, FALSE, node);
+	}
+      }
+      else if (info[0] && !strcmp (info[0], "genflags")) {
+	if (s->game && !g_strcasecmp (s->game, "generations")) {
+	  show_extended_flags (info[1], q3_generations_genflags, Q3_GENERATIONS_GENFLAGS, FALSE, node);
+	}
       }
       break;
-#endif
+    
+    case WO_SERVER:
+      if (info[0] && !g_strcasecmp (info[0], "g_voteFlags"))
+      {
+	  show_extended_flags (info[1], rtcw_voteflags, RTCW_VOTEFLAGS, FALSE, node);
+      }
+      break;
+
+    case WOET_SERVER:
+      if (info[0] && !g_strcasecmp (info[0], "voteflags"))
+      {
+	  show_extended_flags (info[1], woet_voteflags, WOET_VOTEFLAGS, FALSE, node);
+      }
+      break;
+
 
     default:
       break;
