@@ -16,15 +16,16 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gnuconfig.h"
+
 #include <sys/types.h>
 #include <sys/socket.h>	/* inet_ntoa */
 #include <netinet/in.h>	/* inet_ntoa */
 #include <arpa/inet.h>	/* inet_ntoa */
 #include <string.h>	/* strncpy */
 
-#include <gtk/gtk.h>
-
-#include "xqf.h"
+#include "i18n.h"
+#include "xqf-ui.h"
 #include "game.h"
 #include "sort.h"
 #include "skin.h"
@@ -40,6 +41,8 @@
 #include "pixmaps.h"
 #include "srv-info.h"
 #include "srv-list.h"
+#include "srv-prop.h" /*pulp*/
+#include "country-filter.h"
 
 
 GSList *qw_colors_pixmap_cache = NULL;
@@ -53,7 +56,7 @@ static void get_server_pixmap (GtkWidget *window, struct server *s,
                           GSList **cache, GdkPixmap **pix, GdkBitmap **mask) {
   unsigned key;
 
-  if (!s || !window || !buddy_pix[1].pix || !q_pix.pix) {
+  if (!s || !window || !buddy_pix[1].pix) {
     *pix = NULL;
     *mask = NULL;
     return;
@@ -74,7 +77,7 @@ static void get_server_pixmap (GtkWidget *window, struct server *s,
 }
 
 
-void assemble_server_address (char *buf, int size, struct server *s) {
+void assemble_server_address (char *buf, int size, const struct server *s) {
   if (show_default_port || games[s->type].default_port != s->port) {
     g_snprintf (buf, size, "%s:%d", (show_hostnames && s->host->name)? 
                                     s->host->name : inet_ntoa (s->host->ip), 
@@ -90,11 +93,21 @@ void assemble_server_address (char *buf, int size, struct server *s) {
 static int server_clist_refresh_row (struct server *s, int row) {
   GdkPixmap *server_pixmap;
   GdkBitmap *server_pixmask;
-  char *text[10];
-  char buf1[256], buf2[32], buf3[32], buf4[32], buf5[4];
+  char *text[9];
+#ifdef USE_GEOIP
+  struct pixmap* countrypix = NULL;
+#endif
+  
+  char buf1[256], buf2[32], buf3[32], buf4[32];
   char *retries;
   struct pixmap *retries_pix = NULL;
   int col;
+//  char* private_col_text = "";
+  int slots_buffer;
+  unsigned short players = 0;
+  
+
+  struct server_props *p;
 
   text[0] = NULL;
 
@@ -138,20 +151,23 @@ static int server_clist_refresh_row (struct server *s, int row) {
 
   text[3] = text[4] = NULL;
 
-  g_snprintf (buf4, 32, "%d/%d", s->curplayers, s->maxplayers);
-  text[5] = (!s->curplayers)? buf4 : NULL;
+  players = s->curplayers;
+  if(serverlist_countbots && s->curbots <= players)
+	  players-=s->curbots;
+
+  if(s->private_client)
+    g_snprintf (buf4, 32, "%d/%d(-%d)", players, s->maxplayers,s->private_client);
+  else
+    g_snprintf (buf4, 32, "%d/%d", players, s->maxplayers);
+
+  // set text only if no players are on the server. Otherwise an icon is added
+  // later together with this text
+  text[5] = (!players)? buf4 : NULL;
 
   text[6] = (s->map) ?  s->map : NULL;
   text[7] = (s->game)? s->game : NULL;
   text[8] = (s->gametype) ? s->gametype : NULL; 
-  
-  /*
-    baa- Change xqf-ui.c such that the server_clist_def calls out
-    10 columns if you want this shown.
-  */
-  sprintf (buf5, "[%c]", ( s->sv_os ? s->sv_os : ' '));
-  text[9] = buf5;
-
+	  
   if (row < 0) {
     row = gtk_clist_append (server_clist, text);
   }
@@ -170,26 +186,55 @@ static int server_clist_refresh_row (struct server *s, int row) {
      (s->curplayers >= s->maxplayers)? man_red_pix.mask : man_black_pix.mask);
   }
 #endif
-  if (s->curplayers >= s->maxplayers)
+
+   /*pulp*/
+   
+   p = properties (s);
+   
+   if (p) {
+  	  if (p->reserved_slots) {
+
+
+		slots_buffer=p->reserved_slots;
+    	  }
+
+    	  else {
+    		slots_buffer=0;
+    	  }
+    }
+
+    else {
+    slots_buffer=0;
+    }
+
+  if (players >= (s->maxplayers-slots_buffer))
     gtk_clist_set_pixtext (server_clist, row, 5, buf4, 2,
                            man_red_pix.pix, man_red_pix.mask );
 
-  else if ( (s->curplayers + s->private_client ) >= s->maxplayers)
+  else if ( (players + s->private_client ) >= s->maxplayers)
     gtk_clist_set_pixtext (server_clist, row, 5, buf4, 2,
                            man_yellow_pix.pix, man_yellow_pix.mask );
   
-  else if (s->curplayers)
+  else if (players)
     gtk_clist_set_pixtext (server_clist, row, 5, buf4, 2,
                            man_black_pix.pix, man_black_pix.mask );
   
     
 
-  /* Show if the server is private or not */
-  if (s->flags & SERVER_PASSWORD ) {
-    gtk_clist_set_pixtext (server_clist, row, 4, "", 0,
+  /* Show if the server is punkbuster and private or not */
+  if ( (s->flags & SERVER_PUNKBUSTER) && (s->flags & SERVER_PASSWORD) )
+  {
+    gtk_clist_set_pixmap (server_clist, row, 4, 
+                           locked_punkbuster_pix.pix, locked_punkbuster_pix.mask );
+  }
+  
+  else if (s->flags & SERVER_PASSWORD ) {
+    gtk_clist_set_pixmap (server_clist, row, 4, 
                            locked_pix.pix, locked_pix.mask );
-  } else {
-    gtk_clist_set_text (server_clist, row, 4, "" );
+  } 
+  else if (s->flags & SERVER_PUNKBUSTER) {
+    gtk_clist_set_pixmap (server_clist, row, 4, 
+                           punkbuster_pix.pix, punkbuster_pix.mask );
   }
 
   get_server_pixmap (main_window, s, &server_pixmap_cache, &server_pixmap, 
@@ -197,6 +242,28 @@ static int server_clist_refresh_row (struct server *s, int row) {
   gtk_clist_set_pixtext (server_clist, row, 0, 
                    (s->name)? s->name : "", 2, server_pixmap, server_pixmask);
 
+#ifdef USE_GEOIP
+  countrypix = get_pixmap_for_country(s->country_id);
+  if(countrypix)
+	  gtk_clist_set_pixtext (server_clist, row, 1, 
+                   text[1], 2, countrypix->pix, countrypix->mask);
+#endif
+
+  // if map not available
+  if(games[s->type].has_map && games[s->type].has_map(s) == FALSE)
+  {
+    /*
+    GtkStyle *style;
+
+    style = gtk_widget_get_style(GTK_WIDGET(server_clist));
+    if(style)
+    {
+      gtk_clist_set_foreground(server_clist,row,&style->fg[GTK_STATE_INSENSITIVE]);
+    }
+    */
+    gtk_clist_set_pixtext (server_clist, row, 6, s->map, 2, rminus_pix.pix, rminus_pix.mask );
+  }
+
   gdk_pixmap_unref (server_pixmap);
   gdk_bitmap_unref (server_pixmask);
 
@@ -223,7 +290,7 @@ static int player_clist_refresh_row (struct server *s, struct player *p,
   g_snprintf (buf1, 32, "%d", p->frags);
   text[1] = buf1;
 
-  if (p->model && p->skin) {
+  if (p->model && p->skin && strlen(p->model) && strlen(p->skin)) {
     g_snprintf (skin_buf, 128, "%s/%s", p->model, p->skin);
     text[3] = skin_buf;
   }
@@ -335,6 +402,9 @@ void player_clist_redraw (void) {
 void server_clist_sync_selection (void) {
   GList *selection;
 
+  if(!server_clist)
+      return;
+
   debug (7, "server_clist_sync_selection() --");
   if (!sync_selection_blocked) {
     selection = server_clist->selection;
@@ -405,6 +475,7 @@ void server_clist_select_one (int row) {
   sync_selection_blocked = FALSE;
 
   gtk_clist_select_row (server_clist, row, -1);
+  server_clist_selection_visible();
 }
 
 
@@ -434,24 +505,29 @@ GSList *server_clist_all_servers (void) {
   struct server *server;
   int row;
 
-  debug (6, "server_clist_all_servers() -- ");
+  debug (6, "start");
   for (row = 0; row < server_clist->rows; row++) {
     server = (struct server *) gtk_clist_get_row_data (server_clist, row);
     list = server_list_prepend (list, server);
   }
-  debug (7, "server_clist_all_servers() -- Return list %lx", list);
+  debug (6, "Return list %lx", list);
   return g_slist_reverse (list);
 }
 
 
 void server_clist_selection_visible (void) {
-  GList *rows = server_clist->selection;
+  GList *rows;
   GList *row;
   GtkVisibility vis;
   int min;
 
   debug (7, "server_clist_selection_visible() -- ");
 
+  if(!server_clist)
+      return;
+
+  rows = server_clist->selection;
+
   if (!rows)
     return;
 
@@ -559,16 +635,22 @@ void server_clist_set_list (GSList *servers) {
 }
 
 
+/**
+  filter server_list through server filter and display result in clist
+  */
 void server_clist_build_filtered (GSList *server_list, int update) {
   /* This gets called whenever a user clicks the filter button */
 
-  GSList *delete;
-  GSList *add;
-  GSList *tmp;
-  struct server *server;
+//  GSList *delete;
+//  GSList *add;
+//  GSList *tmp;
+//  struct server *server;
   int row;
 
-  debug (6, "server_clist_build_filterd() -- Update? %d", update);
+  debug(3, "server_clist_build_filterd() -- Update? %d", update);
+
+#if 0
+
   delete = server_clist_all_servers ();
   add = build_filtered_list (cur_filter, server_list); /* in filter.c */
 
@@ -637,8 +719,28 @@ void server_clist_build_filtered (GSList *server_list, int update) {
     gtk_clist_clear(server_clist);
 
   gtk_clist_sort (server_clist);
+#else
+
+  {
+
+    struct server* saved_cur_server = cur_server;
+    server_ref(saved_cur_server);
+
+    server_clist_set_list(server_list);
+
+    row = gtk_clist_find_row_from_data (server_clist, saved_cur_server);
+    server_unref(saved_cur_server);
+    saved_cur_server = NULL;
+
+    if (row >= 0)
+      server_clist_select_one (row);
+
+  }
+
+#endif
+
+  
   server_clist_selection_visible ();
 
   gtk_clist_thaw (server_clist);
 }
-
