@@ -26,9 +26,8 @@
 #include <arpa/inet.h>	/* inet_ntoa */
 
 
-#include <gtk/gtk.h>
-
-#include "xqf.h"
+#include "i18n.h"
+#include "xqf-ui.h"
 #include "game.h"
 #include "pref.h"
 #include "server.h"
@@ -62,13 +61,14 @@ struct server_stats {
 struct arch_stats {
   int oscpu[OS_NUM][CPU_NUM];
   int count;
+  int notebookpage; // page in notebook
 };
 
 
 static const char *srv_headers[6] = {
   //The space behind up and down is to make the strings different from
   //those in flt-player.c because their meaning is different in german
-  N_("Servers"), N_("Up "), N_("T/O"), N_("Down "), N_("Info n/a"), N_("Players")
+  N_("Servers"), N_("Up "), N_("Timeout"), N_("Down "), N_("Info n/a"), N_("Players")
 };
 
 static const char *os_names[OS_NUM] = {
@@ -86,17 +86,27 @@ const char *arch_label = N_("OS/CPU");
 static struct server_stats *srv_stats;
 static struct arch_stats *srv_archs;
 
+struct players_s
+{
+  int on_os[OS_NUM];
+  int total;
+};
+
+static struct players_s* players;
+
 static int servers_count;
 static int players_count;
 
 static GtkWidget *stat_notebook;
 static GtkWidget *arch_notebook;
+static enum server_type selected_type;
 
 
 static void server_stats_create (void) {
 
   srv_stats = g_malloc0 (sizeof (struct server_stats) * GAMES_TOTAL);
-  srv_archs  = g_malloc0 (sizeof (struct arch_stats) * 5);
+  srv_archs  = g_malloc0 (sizeof (struct arch_stats) * GAMES_TOTAL);
+  players  = g_malloc0 (sizeof (struct arch_stats) * GAMES_TOTAL);
 
   servers_count = 0;
   players_count = 0;
@@ -115,7 +125,7 @@ static void server_stats_destroy (void) {
 }
 
 
-static enum CPU identify_cpu (struct server *s, const char *versionstr) {
+enum CPU identify_cpu (struct server *s, const char *versionstr) {
   enum CPU cpu = CPU_UNKNOWN;
   char *str;
   
@@ -132,7 +142,7 @@ static enum CPU identify_cpu (struct server *s, const char *versionstr) {
   else if (strstr (str, "ppc"))
     cpu = CPU_PPC;
   else {
-    debug (3, "identify_cpu() -- [%s:%d] Unknown CPU: %s\n", 
+    debug (3, "identify_cpu() -- [%s %s:%d] Unknown CPU: %s\n", type2id(s->type), 
                                 inet_ntoa (s->host->ip), s->port, versionstr);
   }
   g_free(str);
@@ -140,7 +150,7 @@ static enum CPU identify_cpu (struct server *s, const char *versionstr) {
 }
 
 
-static enum OS identify_os (struct server *s, char *versionstr) {
+enum OS identify_os (struct server *s, char *versionstr) {
   enum OS os = OS_UNKNOWN;
   char *str;
   
@@ -157,13 +167,20 @@ static enum OS identify_os (struct server *s, char *versionstr) {
   else if (strstr (str, "macos"))
     os = OS_MACOS;
   else {
-    debug (3, "identify_os() -- [%s:%d] Unknown OS: %s\n", 
+    debug (3, "identify_os() -- [%s %s:%d] Unknown OS: %s\n", type2id(s->type),
                                 inet_ntoa (s->host->ip), s->port, versionstr);
   }
   g_free(str);
   return os;
 }
 
+enum OS t2_identify_os (struct server *s, char *versionstr)
+{
+  if (!strcmp(versionstr,"1"))
+    return OS_LINUX;
+
+  return OS_WINDOWS;
+}
 
 static void collect_statistics (void) {
   GSList *servers;
@@ -172,6 +189,7 @@ static void collect_statistics (void) {
   char **info;
   enum OS os;
   enum CPU cpu;
+  int countthisserver;
 
   servers = all_servers (); /* Free at end of this function */
 
@@ -179,6 +197,9 @@ static void collect_statistics (void) {
     for (tmp = servers; tmp; tmp = tmp->next) {
       s = (struct server *) tmp->data;
       info = s->info;
+      cpu = CPU_UNKNOWN;
+      os = OS_UNKNOWN;
+      countthisserver=0;
 
       servers_count++;
 
@@ -189,7 +210,10 @@ static void collect_statistics (void) {
 
       if (s->ping < MAX_PING) {
 	if (s->ping >= 0)
+	{
 	  srv_stats[s->type].ok++;
+	  countthisserver=1;
+	}
 	else
 	  srv_stats[s->type].na++;
       }
@@ -200,62 +224,35 @@ static void collect_statistics (void) {
 	  srv_stats[s->type].down++;
       }
 
-#ifdef QSTAT23
-      if (info && (s->type == Q2_SERVER || s->type == Q3_SERVER || s->type == WO_SERVER || s->type == KP_SERVER)) {
-#else
-      if (info && (s->type == Q2_SERVER)) {
-#endif 
+      if (info && games[s->type].arch_identifier)
+      {
 	while (info[0]) {
-	  if (g_strcasecmp (info[0], "version") == 0) {
+	  if (g_strcasecmp (info[0], games[s->type].arch_identifier) == 0)
+	  {
 	    if (!info[1])
 	      break;
 
-	    cpu = identify_cpu (s, info[1]); 
-	    os = identify_os (s, info[1]); 
+	    if (games[s->type].identify_cpu)
+	      cpu = games[s->type].identify_cpu(s, info[1]); 
+	    else
+	      cpu = CPU_UNKNOWN;
 
-	    switch (s->type) {
-
-	    case Q2_SERVER:
-	      srv_archs[0].oscpu[os][cpu]++;
-	      srv_archs[0].count++;
-	      break;
-
-
-#ifdef QSTAT23
-	    case Q3_SERVER:
-	      srv_archs[1].oscpu[os][cpu]++;
-	      srv_archs[1].count++;
-	      break;
-
-	    case WO_SERVER:
-	      srv_archs[2].oscpu[os][cpu]++;
-	      srv_archs[2].count++;
-	      break;
-#endif 
-	    case KP_SERVER:
-	      srv_archs[3].oscpu[os][cpu]++;
-	      srv_archs[3].count++;
-	      break;
-
-	    default:
-	      break;
-	    }
+	    if (games[s->type].identify_os)
+	      os = games[s->type].identify_os(s, info[1]); 
+	    else
+	      os = OS_UNKNOWN;
 
 	    break;
 	  }
 	  info += 2;
 	}
-      } else if (info && (s->type == HL_SERVER)) {
-        while (info[0]) {
-	  if (g_strcasecmp (info[0], "sv_os") == 0) {
-	    if (!info[1])
-	      break;
-	    cpu = CPU_UNKNOWN;
-	    os = identify_os (s, info[1]);
-	    srv_archs[4].oscpu[os][cpu]++;
-	    srv_archs[4].count++;
-	  }
-	  info += 2;
+
+	if(countthisserver)
+	{
+	  srv_archs[s->type].oscpu[os][cpu]++;
+	  srv_archs[s->type].count++;
+	  players[s->type].on_os[os] += s->curplayers;
+	  players[s->type].total += s->curplayers;
 	}
       }
     }
@@ -317,23 +314,24 @@ static void put_server_stats (GtkWidget *table, int num, int row) {
 static GtkWidget *server_stats_page (void) {
   GtkWidget *page_vbox;
   GtkWidget *alignment;
-  GtkWidget *frame;
   GtkWidget *table;
   GtkWidget *game_label;
+  GtkWidget *scrollwin;
   int i;
+  int row = 0;
 
   page_vbox = gtk_vbox_new (FALSE, 4);
   gtk_container_set_border_width (GTK_CONTAINER (page_vbox), 8);
 
-  alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-  gtk_box_pack_start (GTK_BOX (page_vbox), alignment, TRUE, TRUE, 0);
+  scrollwin = gtk_scrolled_window_new (NULL, NULL);
+  gtk_box_pack_start (GTK_BOX (page_vbox), scrollwin, TRUE, TRUE, 0);
 
-  frame = gtk_frame_new (NULL);
-  gtk_container_add (GTK_CONTAINER (alignment), frame);
+  alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollwin), alignment);
 
   table = gtk_table_new (GAMES_TOTAL + 2, 7, FALSE);
+  gtk_container_add (GTK_CONTAINER (alignment), table);
   gtk_container_set_border_width (GTK_CONTAINER (table), 6);
-  gtk_container_add (GTK_CONTAINER (frame), table);
 
   gtk_table_set_row_spacings (GTK_TABLE (table), 4);
   gtk_table_set_col_spacings (GTK_TABLE (table), 8);
@@ -347,13 +345,18 @@ static GtkWidget *server_stats_page (void) {
   for (i = 0; i < 6; i++)
     put_label_to_table (table, _(srv_headers[i]), 1.0, i + 1, 0);
 
-  for (i = 0; i < GAMES_TOTAL; i++) {
+  for (i = 0, row = 1; i < GAMES_TOTAL; i++) {
+  
+    // Skip a game if it's not configured and show only configured is enabled
+    if (!games[i].cmd && default_show_only_configured_games)
+      continue;
+              
     game_label = game_pixmap_with_label (i);
     gtk_table_attach_defaults (GTK_TABLE (table), 
 				 game_label, 
-				 0, 1, i + 1, i + 2);
+				 0, 1, row, row+1);
 
-    put_server_stats (table, i, i + 1);
+    put_server_stats (table, i, row);
 
     if (i > 0) {
       srv_stats[0].servers += srv_stats[i].servers;
@@ -363,14 +366,16 @@ static GtkWidget *server_stats_page (void) {
       srv_stats[0].na      += srv_stats[i].na;
       srv_stats[0].players += srv_stats[i].players;
     }
+    
+    row++;
   }
 
-  put_label_to_table (table, _("Total"), 0.0, 0, GAMES_TOTAL + 1);
+  put_label_to_table (table, _("Total"), 0.0, 0, row + 1);
 
-  put_server_stats (table, 0, GAMES_TOTAL + 1);
+  put_server_stats (table, 0, row + 1);
 
   gtk_widget_show (table);
-  gtk_widget_show (frame);
+  gtk_widget_show (scrollwin);
 
   gtk_widget_show (alignment);
   gtk_widget_show (page_vbox);
@@ -394,10 +399,9 @@ static void arch_notebook_page (GtkWidget *notebook,
   int i, j;
   int cpu_total;
 
-  table = gtk_table_new (OS_NUM + 2, CPU_NUM + 2, FALSE);
+  table = gtk_table_new (OS_NUM + 2, CPU_NUM + 3, FALSE);
 
-  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, 
-                                               game_pixmap_with_label (type));
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, NULL);
 
   gtk_container_set_border_width (GTK_CONTAINER (table), 6);
   gtk_table_set_row_spacings (GTK_TABLE (table), 4);
@@ -407,6 +411,7 @@ static void arch_notebook_page (GtkWidget *notebook,
   gtk_table_set_row_spacing (GTK_TABLE (table), 0, 12);
   gtk_table_set_col_spacing (GTK_TABLE (table), OS_NUM, 20);
   gtk_table_set_row_spacing (GTK_TABLE (table), CPU_NUM, 12);
+  gtk_table_set_row_spacing (GTK_TABLE (table), CPU_NUM+1, 12);
 
   put_label_to_table (table, _("CPU  \\  OS"), 0.5, 0, 0);
 
@@ -419,6 +424,7 @@ static void arch_notebook_page (GtkWidget *notebook,
     put_label_to_table (table, _(cpu_names[i]), 0.0, 0, i + 1);
   }
   put_label_to_table (table, _("Total"), 0.0, 0, CPU_NUM + 1);
+  put_label_to_table (table, _("Players"), 0.0, 0, CPU_NUM + 2);
 
   for (j = 0; j < CPU_NUM; j++) {
     cpu_total = 0;
@@ -435,47 +441,79 @@ static void arch_notebook_page (GtkWidget *notebook,
 
   for (i = 0; i < OS_NUM; i++) {
     put_arch_stats (table, arch->oscpu[i][0], arch->count, i + 1, CPU_NUM + 1);
+    put_arch_stats (table, players[type].on_os[i],
+	players[type].total, i + 1, CPU_NUM + 2);
   }
   put_arch_stats (table, arch->count, arch->count, OS_NUM + 1, CPU_NUM + 1);
+  put_arch_stats (table, srv_stats[type].players, players_count, OS_NUM + 1, CPU_NUM + 2);
 
   gtk_widget_show (table);
 }
 
+gboolean create_server_type_menu_filter_hasharch(enum server_type type)
+{
+    if ((!games[type].cmd && default_show_only_configured_games)
+	  || !games[type].arch_identifier)
+	return FALSE;
+    else
+	return TRUE;
+}
+
+static void select_server_type_callback(GtkWidget *widget, enum server_type type)
+{
+  gtk_notebook_set_page (GTK_NOTEBOOK (arch_notebook), srv_archs[type].notebookpage );
+  selected_type = type;
+}
 
 static GtkWidget *archs_stats_page (void) {
   GtkWidget *page_vbox;
   GtkWidget *alignment;
-  char *typestr;
+  GtkWidget *option_menu;
+  GtkWidget *hbox;
+  int pagenum = 0;
   enum server_type type = Q2_SERVER;
+  enum server_type to_activate = UNKNOWN_SERVER;
 
   page_vbox = gtk_vbox_new (FALSE, 4);
   gtk_container_set_border_width (GTK_CONTAINER (page_vbox), 8);
 
+
   alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-  gtk_box_pack_start (GTK_BOX (page_vbox), alignment, TRUE, TRUE, 0);
 
   arch_notebook = gtk_notebook_new ();
+  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (arch_notebook), FALSE);
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (arch_notebook), GTK_POS_TOP);
   gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (arch_notebook), 4);
+  gtk_notebook_set_show_border(GTK_NOTEBOOK(arch_notebook), FALSE);
   gtk_container_add (GTK_CONTAINER (alignment), arch_notebook);
 
-  arch_notebook_page (arch_notebook, Q2_SERVER, &srv_archs[0]);
-#ifdef QSTAT23
-  arch_notebook_page (arch_notebook, Q3_SERVER, &srv_archs[1]);
-  arch_notebook_page (arch_notebook, WO_SERVER, &srv_archs[2]);
-#endif 
-  arch_notebook_page (arch_notebook, KP_SERVER, &srv_archs[3]);
-  arch_notebook_page (arch_notebook, HL_SERVER, &srv_archs[4]);
-#ifdef QSTAT23
-  typestr = config_get_string ("/" CONFIG_FILE "/Statistics/game");
-  if (typestr) {
-    type = id2type (typestr);
-    g_free (typestr);
+  to_activate = config_get_int("/" CONFIG_FILE "/Statistics/game");
+
+  for (type = 0; type < GAMES_TOTAL; ++type)
+  {
+    if(!create_server_type_menu_filter_hasharch(type))
+      continue;
+
+    srv_archs[type].notebookpage=pagenum++;
+
+    arch_notebook_page (arch_notebook, type, &srv_archs[type]);
   }
 
-  gtk_notebook_set_page (GTK_NOTEBOOK (arch_notebook), 
-                                                  (type == Q2_SERVER)? 0 : 1);
-#endif 
+  // the notebook must exist to allow activate events of the menu
+  hbox = gtk_hbox_new(FALSE,0);
+  gtk_box_pack_start (GTK_BOX (page_vbox), hbox, FALSE, TRUE, 0);
+
+    option_menu = create_server_type_menu (to_activate,
+		    create_server_type_menu_filter_hasharch,
+		    GTK_SIGNAL_FUNC(select_server_type_callback));
+
+
+    gtk_box_pack_start (GTK_BOX (hbox), option_menu, TRUE, FALSE, 0);
+    gtk_widget_show (option_menu);
+
+  gtk_widget_show(hbox);
+
+  gtk_box_pack_start (GTK_BOX (page_vbox), alignment, TRUE, TRUE, 0);
 
   gtk_widget_show (arch_notebook);
   gtk_widget_show (alignment);
@@ -485,23 +523,36 @@ static GtkWidget *archs_stats_page (void) {
 }
 
 
-static void grab_defaults (GtkWidget *w, gpointer data) {
-  enum server_type type;
-
-#ifdef QSTAT23
-  type = (gtk_notebook_get_current_page (GTK_NOTEBOOK (arch_notebook)) == 0) ?
-                                                        Q2_SERVER : Q3_SERVER;
-#else
-  type = Q2_SERVER;
-#endif
-
-  config_set_string ("/" CONFIG_FILE "/Statistics/game", type2id (type));
+static void grab_defaults (GtkWidget *w, gpointer data)
+{
+  config_set_int ("/" CONFIG_FILE "/Statistics/game", selected_type);
 
   config_set_string ("/" CONFIG_FILE "/Statistics/page", 
         (gtk_notebook_get_current_page (GTK_NOTEBOOK (stat_notebook)) == 0) ?
                                                       srv_label : arch_label);
 }
 
+static void statistics_save_geometry (GtkWidget *window, gpointer data) {
+  config_push_prefix ("/" CONFIG_FILE "/Statistics Window Geometry/");
+  config_set_int ("height", window->allocation.height);
+  config_set_int ("width", window->allocation.width);
+  config_pop_prefix ();
+}
+
+static void statistics_restore_geometry (GtkWidget *window) {
+  int height, width;
+
+  config_push_prefix ("/" CONFIG_FILE "/Statistics Window Geometry/");
+
+  height = config_get_int ("height");
+  width = config_get_int ("width");
+  if(!width || !height)
+    return;
+
+  gtk_window_set_default_size (GTK_WINDOW (window), width, height);
+
+  config_pop_prefix ();
+}
 
 void statistics_dialog (void) {
   GtkWidget *window;
@@ -517,7 +568,10 @@ void statistics_dialog (void) {
   collect_statistics ();
 
   window = dialog_create_modal_transient_window (_("Statistics"), 
-                                                           TRUE, FALSE, NULL);
+                                                           TRUE, TRUE, statistics_save_geometry);
+
+  statistics_restore_geometry(window);
+
   main_vbox = gtk_vbox_new (FALSE, 8);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 8);
   gtk_container_add (GTK_CONTAINER (window), main_vbox);
@@ -529,7 +583,7 @@ void statistics_dialog (void) {
   stat_notebook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (stat_notebook), GTK_POS_TOP);
   gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (stat_notebook), 4);
-  gtk_box_pack_start (GTK_BOX (main_vbox), stat_notebook, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (main_vbox), stat_notebook, TRUE, TRUE, 0);
 
   page = server_stats_page ();
   label = gtk_label_new (_(srv_label));
@@ -577,4 +631,3 @@ void statistics_dialog (void) {
 
   server_stats_destroy ();
 }
-
