@@ -21,6 +21,7 @@
 
 #include <sys/types.h>
 #include <stdio.h>	/* FILE */
+#include <time.h>
 
 #include <glib.h>
 
@@ -40,6 +41,9 @@ extern	GList *merge_sorted_string_lists (GList *list1, GList *list2);
 
 extern	GSList *unique_strings (GSList *strings);
 
+// build GList from array of char*
+extern GList* createGListfromchar(char* strings[]);
+  
 extern	void on_sig (int signum, void (*func) (int signum));
 extern 	void ignore_sigpipe (void);
 
@@ -54,6 +58,11 @@ extern	int tokenize_bychar (char *str, char *token[], int max, char dlm);
 
 extern	int hostname_is_valid (const char *hostname);
 
+extern	char *find_game_dir (const char *basegamedir, const char *game,
+						int *match_result);
+
+extern	char* resolve_path(const char* path);
+
 /* 
    Find a server setting from the info list in 
    the server struct.  The key passed will be converted to lower case.
@@ -61,5 +70,30 @@ extern	int hostname_is_valid (const char *hostname);
 extern char* find_server_setting_for_key (char*, char**);
 
 
+// return "false" if i == 0, "true" otherwise
+const char* bool2str(int i);
+
+// returns true if str is "true", false otherwise
+int str2bool(const char* str);
+
+
+/** find executable file in $PATH. file is a colon seperated list of
+ * executables to search for. return name of first found file, must be freed
+ * manually
+ */
+char* find_file_in_path(const char* files);
+
+/** sort list and remove duplicates
+ * @param list list to sort
+ * @compare_func function to use for comparing
+ * @unref_func function to call for each deleted entry
+ * @return sorted list without duplicates
+ */
+GSList* slist_sort_remove_dups(GSList* list, GCompareFunc compare_func, void (*unref_func)(void*));
+
+/**
+ * return locale's string representation of t. must be freed manually
+ */
+char* timet2string(const time_t* t);
 
 #endif /* __UTILS_H__ */
