@@ -16,12 +16,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gnuconfig.h"
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdarg.h>	/* va_start, va_end */
 
 #include <gtk/gtk.h>
 
+#include "i18n.h"
 #include "xqf.h"
 #include "xqf-ui.h"
 #include "srv-list.h"
@@ -29,8 +32,8 @@
 #include "game.h"
 #include "config.h"
 #include "sort.h"
-#include "gnuconfig.h"
-
+#include "pref.h"
+#include "debug.h"
 
 static GSList *xqf_windows = NULL;
 static GtkWidget *target_window = NULL;
@@ -42,12 +45,13 @@ GtkWidget *pane3_widget;
 /* If you add a column here to appear in the server
    list, you need to also add an entry in sort.h and sort.c 
 */
+
 static struct clist_column server_columns[10] = {
   { N_("Name"),    180,  GTK_JUSTIFY_LEFT,   NULL },
   { N_("Address"), 140,  GTK_JUSTIFY_LEFT,   NULL },
   { N_("Ping"),     45,  GTK_JUSTIFY_RIGHT,  NULL },
   { N_("TO"),       35,  GTK_JUSTIFY_RIGHT,  NULL },
-  { N_("Priv"),     20,  GTK_JUSTIFY_RIGHT,  NULL },
+  { N_("Priv"),     35,  GTK_JUSTIFY_RIGHT,  NULL },
   { N_("Players"),  65,  GTK_JUSTIFY_RIGHT,  NULL },
   { N_("Map"),      55,  GTK_JUSTIFY_LEFT,   NULL },
   { N_("Game"),     55,  GTK_JUSTIFY_LEFT,   NULL },
@@ -55,6 +59,7 @@ static struct clist_column server_columns[10] = {
   { N_("OS"),       35,  GTK_JUSTIFY_LEFT,   NULL }
 };
 
+
 struct clist_def server_clist_def = {
   CWIDGET_CLIST,
   "Server List",
@@ -136,6 +141,7 @@ int window_delete_event_callback (GtkWidget *widget, gpointer data) {
 
 void register_window (GtkWidget *window) {
   xqf_windows = g_slist_prepend (xqf_windows, window);
+  debug(6, "%p", window);
 }
 
 
@@ -150,6 +156,8 @@ void unregister_window (GtkWidget *window) {
     gtk_widget_destroy ((GtkWidget *) (xqf_windows->data));
   else
     target_window = NULL;
+
+  debug(6, "%p", window);
 }
 
 
@@ -347,6 +355,12 @@ void source_ctree_add_master (GtkWidget *ctree, struct master *m) {
   if (m->isgroup)
     return;
 
+  // If set to display only configured games, and game is not configured,
+  // and it's not the 'Favorites' master, just return so the display
+  // isn't updated with the game type and masters.
+  if (!(games[m->type].cmd) && default_show_only_configured_games && m != favorites)
+    return;
+  
   if (m->type != UNKNOWN_SERVER) {
     group = (struct master *) g_slist_nth_data (master_groups, m->type);
     source_ctree_enable_master_group (ctree, group, TRUE);
@@ -366,7 +380,8 @@ void source_ctree_add_master (GtkWidget *ctree, struct master *m) {
 }
 
 
-static void source_ctree_remove_master_group (GtkWidget *ctree,
+//static void source_ctree_remove_master_group (GtkWidget *ctree,
+void source_ctree_remove_master_group (GtkWidget *ctree,
 					      struct master *m) {
   GtkCTreeNode *node;
 
@@ -415,15 +430,20 @@ static void fill_source_ctree (GtkWidget *ctree) {
   for (list = master_groups; list; list = list->next) {
     group = (struct master *) list->data;
     if (group->masters) {
-      source_ctree_enable_master_group (ctree, group, FALSE);
-      parent = gtk_ctree_find_by_row_data (GTK_CTREE (ctree), NULL, group);
+      // If set to display only configured games, and game is not configured,
+      // don't update the display with the master.
+      if (games[group->type].cmd || !default_show_only_configured_games)
+      {
+        source_ctree_enable_master_group (ctree, group, FALSE);
+        parent = gtk_ctree_find_by_row_data (GTK_CTREE (ctree), NULL, group);
 
-      for (list2 = group->masters; list2; list2 = list2->next) {
-	m = (struct master *) list2->data;
-	node = gtk_ctree_insert_node (GTK_CTREE (ctree), parent, NULL, NULL, 4,
+        for (list2 = group->masters; list2; list2 = list2->next) {
+  	  m = (struct master *) list2->data;
+	  node = gtk_ctree_insert_node (GTK_CTREE (ctree), parent, NULL, NULL, 4,
                                          NULL, NULL, NULL, NULL, TRUE, FALSE);
-	gtk_ctree_node_set_row_data (GTK_CTREE (ctree), node, m);
-	source_ctree_show_node_status (ctree, m);
+	  gtk_ctree_node_set_row_data (GTK_CTREE (ctree), node, m);
+	  source_ctree_show_node_status (ctree, m);
+        }
       }
     }
   }
@@ -474,10 +494,16 @@ int calculate_clist_row_height (GtkWidget *clist, GdkPixmap *pixmap) {
 
   gdk_window_get_size (pixmap, &pix_w, &pix_h);
 
+/*FIXME_GTK2: style->font not working with gtk2*/
+#ifdef USE_GTK2
+  height=pix_h+1;
+#else
+
   height = MAX (pix_h, clist->style->font->ascent +
                                              clist->style->font->descent + 1);
   if (height == pix_h + 1)
     height++;	/* beautify selection */
+#endif
 
   return height;
 }
@@ -485,6 +511,31 @@ int calculate_clist_row_height (GtkWidget *clist, GdkPixmap *pixmap) {
 
 void set_toolbar_appearance (GtkToolbar *toolbar, int style, int tips) {
 
+/*FIXME_GTK2: gtk_toolbar_set_button_relief (), gtk_toolbar_set_space_style () */
+/*gtk_toolbar_set_space_size () is deprecated*/
+#ifdef USE_GTK2
+  switch (style) {
+
+  case 0:
+    gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_ICONS);
+
+    break;
+
+  case 1:
+    gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_TEXT);
+
+    break;
+
+  case 2:
+    gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_BOTH);
+
+    break;
+
+  default:
+    break;
+  }
+
+#else
   switch (style) {
 
   case 0:
@@ -511,6 +562,7 @@ void set_toolbar_appearance (GtkToolbar *toolbar, int style, int tips) {
   default:
     break;
   }
+#endif
 
   gtk_toolbar_set_tooltips (toolbar, tips);
 }
@@ -677,8 +729,8 @@ void restore_main_window_geometry (void) {
 
   config_push_prefix ("/" CONFIG_FILE "/Main Window Geometry/");
 
-  height = config_get_int ("height");
-  width  = config_get_int ("width");
+  height = config_get_int ("height=480");
+  width  = config_get_int ("width=640");
   pane1  = config_get_int ("pane1");
   pane2  = config_get_int ("pane2");
   pane3  = config_get_int ("pane3");
@@ -691,9 +743,106 @@ void restore_main_window_geometry (void) {
   }
 
   gtk_paned_set_position (GTK_PANED (pane1_widget), (pane1)? pane1 : 120);
+
+/*FIXME_GTK2: GTK_PANED ()->gutter_size is not working, do not know why*/
+#ifdef USE_GTK2
+  gtk_paned_set_position (GTK_PANED (pane2_widget), (pane2)? pane2 :
+           server_clist_def.height +4);
+  gtk_paned_set_position (GTK_PANED (pane3_widget), (pane3)? pane3 :
+           player_clist_def.height + 4);
+#else
   gtk_paned_set_position (GTK_PANED (pane2_widget), (pane2)? pane2 :
            server_clist_def.height + GTK_PANED (pane2_widget)->gutter_size/2);
   gtk_paned_set_position (GTK_PANED (pane3_widget), (pane3)? pane3 :
            player_clist_def.height + GTK_PANED (pane3_widget)->gutter_size/2);
+#endif
 }
 
+GtkWidget* lookup_widget (GtkWidget* widget, const gchar* widget_name)
+{
+  GtkWidget *parent, *found_widget;
+
+  for (;;)
+    {
+      if (GTK_IS_MENU (widget))
+        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
+      else
+        parent = widget->parent;
+      if (parent == NULL)
+        break;
+      widget = parent;
+    }
+
+  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
+                                                   widget_name);
+  if (!found_widget)
+    g_warning ("Widget not found: %s", widget_name);
+  return found_widget;
+}
+
+// Skip a game if it's not configured and show only configured is enabled
+gboolean create_server_type_menu_filter_configured(enum server_type type)
+{
+    if (!games[type].cmd && default_show_only_configured_games)
+	return FALSE;
+    else
+	return TRUE;
+}
+
+GtkWidget *create_server_type_menu (enum server_type active_type,
+	gboolean (*filterfunc)(enum server_type),
+	GtkSignalFunc callback)
+{
+  GtkWidget *option_menu = NULL;
+  GtkWidget *menu = NULL;
+  GtkWidget *menu_item = NULL;
+  GtkWidget *first_menu_item = NULL;
+  int i;
+  int j = 0;
+  int menu_type = 0;
+
+  option_menu = gtk_option_menu_new ();
+
+  menu = gtk_menu_new ();
+
+  for (i = 0; i < GAMES_TOTAL; ++i)
+  {
+
+    if(filterfunc && !filterfunc(i))
+      continue;
+
+    menu_item = gtk_menu_item_new ();
+
+    if (i == active_type)
+    {
+      first_menu_item = menu_item;
+      menu_type = j;
+    }
+    else if (i == 0)
+      first_menu_item = menu_item;
+    
+    gtk_menu_append (GTK_MENU (menu), menu_item);
+
+    gtk_container_add (GTK_CONTAINER (menu_item), game_pixmap_with_label (i));
+
+    if(callback)
+      gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+                 GTK_SIGNAL_FUNC (callback), (gpointer) i);
+
+    gtk_widget_show (menu_item);
+
+    ++j; // must be here in case the continue was used
+  }
+
+  // initiates callback to set servertype to first configured game
+  if(first_menu_item)
+      gtk_menu_item_activate (GTK_MENU_ITEM (first_menu_item)); 
+
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
+
+  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), menu_type);
+
+  return option_menu;
+}
+
+
