@@ -16,11 +16,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#ifndef __XQF_UI_H__
-#define __XQF_UI_H__
+#ifndef XQF_UI_H__
+#define XQF_UI_H__
 
 #include <gtk/gtk.h>
 
+#include <xqf.h>
+
 
 struct clist_column {
   const char *name;
@@ -60,13 +62,27 @@ extern	GtkWidget *pane1_widget;
 extern  GtkWidget *pane2_widget;
 extern  GtkWidget *pane3_widget;
 
-extern	void print_status (GtkWidget *sbar, char *fmt, ...);
+
+extern 	GtkWidget *main_window;
+extern	GtkWidget *source_ctree;
+extern  GtkCList  *server_clist;
+extern  GtkCList  *player_clist;
+extern  GtkCTree  *srvinf_ctree;
+
+extern	GtkWidget *view_hostnames_menu_item;
+extern	GtkWidget *view_defport_menu_item;
 
 extern	int window_delete_event_callback (GtkWidget *widget, gpointer data);
 extern	void register_window (GtkWidget *window);
 extern	void unregister_window (GtkWidget *window);
 extern	GtkWidget *top_window (void);
 
+extern	void set_widgets_sensitivity (void);
+
+extern GtkWidget *server_filter_widget[];
+
+extern	void print_status (GtkWidget *sbar, char *fmt, ...);
+
 extern	GtkWidget *create_cwidget (GtkWidget *scrollwin, 
                                                      struct clist_def *cldef);
 
@@ -80,6 +96,7 @@ extern	void source_ctree_show_node_status (GtkWidget *ctree,
 
 extern	void source_ctree_add_master (GtkWidget *ctree, struct master *m);
 extern	void source_ctree_delete_master (GtkWidget *ctree, struct master *m);
+extern	void source_ctree_remove_master_group (GtkWidget *ctree, struct master *m);
 extern	GtkWidget *create_source_ctree (GtkWidget *scrollwin);
 extern	void source_ctree_select_source (struct master *m);
 
@@ -95,5 +112,32 @@ extern	void progress_bar_set_percentage (GtkWidget *pbar, float percentage);
 extern	void ui_done (void);
 extern	void restore_main_window_geometry (void);
 
+extern 	GtkTooltips *tooltips;
 
-#endif /* __XQF_UI_H__ */
+/*
+ * This function returns a widget in a component created by Glade.
+ * Call it with the toplevel widget in the component (i.e. a window/dialog),
+ * or alternatively any widget in the component, and the name of the widget
+ * you want returned.
+ */
+GtkWidget* lookup_widget (GtkWidget* widget, const gchar* widget_name);
+
+
+/**
+  create a GtkOptionMenu that contains all game names
+  @param active_type which game to set active by default
+  @param filterfunc only add game to menu if this function return true. can be
+	  NULL to show all
+  @param callback a callback function to connect if a game entry is activated.
+	  the user_data will be set to the game type
+  @return a GtkOptionMenu
+  */
+GtkWidget *create_server_type_menu (enum server_type active_type,
+	gboolean (*filterfunc)(enum server_type),
+	GtkSignalFunc callback);
+
+
+/** Skip a game if it's not configured and show only configured is enabled */
+gboolean create_server_type_menu_filter_configured(enum server_type type);
+
+#endif /* XQF_UI_H__ */
