@@ -19,14 +19,14 @@
 #ifndef __XQF_H__
 #define __XQF_H__
 
+#include "gnuconfig.h"	/* GNU autoconf */
+
 #include <sys/types.h>
 #include <netinet/in.h> /* struct in_addr */
 #include <arpa/inet.h>	/* struct in_addr */
 #include <time.h>	/* time_t */
 
-#include <gtk/gtk.h>
-
-#include "gnuconfig.h"	/* GNU autoconf */
+#include <glib.h>
 
 #define RC_DIR		".qf"
 #define RC_FILE		"qfrc"
@@ -38,8 +38,6 @@
 #define PASSWORD_CFG	"__passwd.cfg"
 #define LAUNCHINFO_FILE	"LaunchInfo.txt"
 
-#define MAX_SERVER_FILTERS 10
-
 #define MAX_PING	9999
 #define MAX_RETRIES	10
 
@@ -48,15 +46,19 @@
 #define	Q2_DEFAULT_PORT		27910		/* Quake2 */
 #define	Q3_DEFAULT_PORT		27960		/* Quake3: Arena */
 #define	WO_DEFAULT_PORT		27960		/* Wolfenstein */
+#define	EF_DEFAULT_PORT		27960		/* Voyager Elite Force */
 #define	H2_DEFAULT_PORT		26900		/* Hexen2 */
 #define	HW_DEFAULT_PORT		26950		/* HexenWorld */
 #define	SN_DEFAULT_PORT		22450		/* Sin */
 #define	HL_DEFAULT_PORT		27015		/* Half-Life */
 #define	KP_DEFAULT_PORT		31510		/* Kingpin */
 #define	SFS_DEFAULT_PORT	28910		/* Soldier of Fortune */
+#define	SOF2S_DEFAULT_PORT	20100		/* Soldier of Fortune 2 */
 #define	T2_DEFAULT_PORT		28000		/* Tribes 2 */
 #define	HR_DEFAULT_PORT		28910		/* Heretic2 */
 #define	UN_DEFAULT_PORT		7777		/* Unreal */
+#define	AAO_DEFAULT_PORT	1716		/* AAO */
+#define	UT2_DEFAULT_PORT	7777		/* Unreal Tournament 2003 */
 #define	GPS_DEFAULT_PORT	27888		/* Gamespy Generic */
 #define	DESCENT3_DEFAULT_PORT	2092		/* Descent 3 */
 
@@ -66,16 +68,21 @@
 #define	WOM_DEFAULT_PORT	27950		/* Wolfenstein master */
 #define	HLM_DEFAULT_PORT	27010		/* Half-Life master */
 #define	D3M_DEFAULT_PORT	3445		/* Descent 3 master */
+#define	EFM_DEFAULT_PORT	27953		/* Elite Force master */
+#define	T2M_DEFAULT_PORT	28000		/* Tribes 2 */
+#define	SOF2M_DEFAULT_PORT	20110		/* Soldier of Fortune 2 */
 
 #define PLAYER_GROUP_MASK	0x07
 #define PLAYER_GROUP_RED	0x01
 #define PLAYER_GROUP_GREEN	0x02
 #define PLAYER_GROUP_BLUE	0x04
 
+/* max 0x80, server->flags is char */
 #define SERVER_CHEATS		0x08
 #define SERVER_PASSWORD		0x10
 #define SERVER_SP_PASSWORD	0x20
 #define SERVER_SPECTATE		0x40
+#define SERVER_PUNKBUSTER	0x80
 
 enum launch_mode { 
   LAUNCH_NORMAL,
@@ -83,27 +90,38 @@ enum launch_mode {
   LAUNCH_RECORD
 };
 
+
+// note: there is a limit of 256 servers, see get_server_pixmap in srv-list.c
 enum server_type { 
   Q1_SERVER = 0,
   QW_SERVER,
   Q2_SERVER,
-#ifdef QSTAT23
   Q3_SERVER,
-#endif
   WO_SERVER,
+  WOET_SERVER,
+  EF_SERVER,
   H2_SERVER,
   HW_SERVER,
   SN_SERVER,
   HL_SERVER,
   KP_SERVER,
   SFS_SERVER,
+  SOF2S_SERVER,
   T2_SERVER,
   HR_SERVER,
-#ifdef QSTAT_HAS_UNREAL_SUPPORT
   UN_SERVER,
+  UT2_SERVER,
+  UT2004_SERVER,
   RUNE_SERVER,
-#endif
+  POSTAL2_SERVER,
+  AAO_SERVER,
   DESCENT3_SERVER,
+  SSAM_SERVER,
+  SSAMSE_SERVER,
+  MOHAA_SERVER,
+  COD_SERVER,
+  SAS_SERVER,
+  BF1942_SERVER,
   GPS_SERVER,
   UNKNOWN_SERVER
 };
@@ -118,6 +136,21 @@ enum master_state {
   SOURCE_ERROR
 };
 
+enum master_query_type { 
+	MASTER_NATIVE=0,
+	MASTER_GAMESPY,
+	MASTER_HTTP,
+	MASTER_LAN,
+	MASTER_FILE,
+	MASTER_NUM_QUERY_TYPES,
+	MASTER_INVALID_TYPE
+};
+
+
+/**
+ * A player. There is no destructor for this struct. Alloc as many data as you
+ * need at once and have the pointers point inside.
+ */
 struct player {
   char	*name;
   int	time;
@@ -137,32 +170,42 @@ struct host {
   int ref_count;
 };
 
+
+/** server_new, server_free_info, server_move_info must be adapted for
+ * changes to this structure
+ */
 struct server {
+  enum server_type type;
   struct host *host;
   unsigned short port;
 
   char	*name;
   char	*map;
-  char  *gametype;
   unsigned short maxplayers;
   unsigned short curplayers;
+  unsigned short curbots;
+  unsigned short private_client; /** number of private clients */
   short	ping;
   short retries;
   char 	**info;
-  char 	*game;		/* a reference to `info' */
-  GSList *players;	/* GSList<struct player *>  */
+  char 	*game;	  /** pointer into info, do not free */
+  char  *gametype; /** pointer into info, do not free */
+  GSList *players;	/** GSList<struct player *>  */
 
-  char sv_os;         /* L = Linux, W = windows, M = Mac */
+  char sv_os;         /** L = Linux, W = windows, M = Mac */
+	
+#ifdef USE_GEOIP
+  int country_id;
+#endif
 
-  unsigned char type;	/* enum server_type type; */
   unsigned char flags;
 
   unsigned char filters;
   unsigned char flt_mask;
-  unsigned flt_last;	/* time of the last filtering */
-  unsigned private_client; /* number of private clients */
+  unsigned flt_last;	/** time of the last filtering */
 
   time_t refreshed;
+  time_t last_answer; /** time of last reply from server */
 
   int 	ref_count;
 };
@@ -193,16 +236,15 @@ struct master {
 
   GSList *masters;
   
-  int master_type;
+  enum master_query_type master_type;
 };
 
 extern	time_t xqf_start_time;
+extern char* xqf_PACKAGE_DATA_DIR;
+extern char* xqf_LOCALEDIR;
+extern char* xqf_PIXMAPSDIR;
 
-extern 	GtkWidget *main_window;
-extern	GtkWidget *source_ctree;
-extern  GtkCList  *server_clist;
-extern  GtkCList  *player_clist;
-extern  GtkCTree  *srvinf_ctree;
+extern char* qstat_configfile;
 
 extern	GSList *cur_source;		/*  GSList <struct master *>  */
 extern	GSList *cur_server_list;	/*  GSList <struct server *>  */
@@ -212,21 +254,21 @@ extern	struct server *cur_server;
 
 extern	struct stat_job *stat_process;
 
-extern	GtkWidget *view_hostnames_menu_item;
-extern	GtkWidget *view_defport_menu_item;
 
-extern	int window_delete_event_callback (GtkWidget *widget, gpointer data);
-extern	void register_window (GtkWidget *window);
-extern	void unregister_window (GtkWidget *window);
-extern	GtkWidget *top_window (void);
+int compare_qstat_version ( const char* have, const char* expected );
+int start_prog_and_return_fd(char *const argv[], pid_t *pid);
+int check_qstat_version(void);
 
-extern	void set_widgets_sensitivity (void);
+extern	void play_sound (const char *sound, const int override);
 
+extern	void reset_main_status_bar(void);
 
-/* This is defined in here so that filter.c can see it */
-GtkWidget *server_filter_widget[MAX_SERVER_FILTERS + 3];
+//extern int filter_start_index;
 
-int filter_start_index;
+extern int event_type;
+
+extern int dontlaunch;
+
+extern	void refresh_source_list (void);
 
 #endif /* __XQF_H__ */
-
